@@ -493,59 +493,190 @@ void xsane_back_gtk_set_option(int opt_num, void *val, SANE_Action action)
 
 /* ----------------------------------------------------------------------------------------------------------------- */
 
-void xsane_back_gtk_close_dialog_callback(GtkWidget * widget, gpointer data)
+int xsane_back_gtk_get_option_double(int option, double *val, SANE_Int *unit)
+/* return values: */
+/*  0 = OK */
+/* -1 = option number < 0 */
+/* -2 = failed to set option */
 {
-  DBG(DBG_proc, "xsane_back_gtk_close_dialog_callback\n");
+ const SANE_Option_Descriptor *opt;
+ SANE_Handle dev;
+ SANE_Word word;
 
-  gtk_widget_destroy(data);
-  xsane.back_gtk_message_dialog_active = 0;
+  DBG(DBG_proc, "xsane_back_gtk_get_option_double\n");
+
+  if (option <= 0)
+  {
+    return -1;
+  }
+
+  if (xsane_control_option(xsane.dev, option, SANE_ACTION_GET_VALUE, &word, 0) == SANE_STATUS_GOOD)
+  {
+    dev = xsane.dev;
+    opt = xsane_get_option_descriptor(dev, option);
+
+    if (unit)
+    {
+      *unit = opt->unit;
+    }
+
+    if (val)
+    {
+      if (opt->type == SANE_TYPE_FIXED)
+      {
+        *val = (float) word / 65536.0;
+      }
+      else
+      {
+        *val = (float) word;
+      }
+    }
+
+   return 0;
+  }
+  else if (val)
+  {
+    *val = 0;
+  }
+  return -2;
 }
 
 /* ----------------------------------------------------------------------------------------------------------------- */
 
-static gint decision_flag;
-static GtkWidget *decision_dialog;
-
-void xsane_back_gtk_decision_callback(GtkWidget * widget, gpointer data)
+int xsane_back_gtk_set_option_double(int option, double value)
 {
-  DBG(DBG_proc, "xsane_back_gtk_decision_callback\n");
+ const SANE_Option_Descriptor *opt;
+ SANE_Word word;
 
-  gtk_widget_destroy(decision_dialog);
-  xsane.back_gtk_message_dialog_active = 0;
-  decision_flag = (long) data;
+  DBG(DBG_proc, "xsane_set_option_double\n");
+
+  if (option <= 0 || value <= -INF || value >= INF)
+  {
+    return -1;
+  }
+
+  opt = xsane_get_option_descriptor(xsane.dev, option);
+  if (opt)
+  {
+    if (opt->type == SANE_TYPE_FIXED)
+    {
+      word = SANE_FIX(value);
+    }
+    else
+    {
+      word = value + 0.5;
+    }
+
+    if (xsane_control_option(xsane.dev, option, SANE_ACTION_SET_VALUE, &word, 0))
+    {
+      return -2;
+    }
+  }
+  else
+  {
+    return -1;
+  }
+
+ return 0;
+}
+
+/* ----------------------------------------------------------------------------------------------------------------- */
+
+static int xsane_back_gtk_decision_delete_event(GtkWidget * widget, GdkEvent *event, gpointer data)
+{
+ gint *decision_flag = (gint *) data;
+
+  DBG(DBG_proc, "xsane_back_gtk_decision_delete_event\n");
+
+  xsane.back_gtk_message_dialog_active--;
+
+  if (decision_flag)
+  {
+    *decision_flag = -1;
+  }
+
+ return FALSE; /* continue with original delete even routine */
+}
+
+/* ----------------------------------------------------------------------------------------------------------------- */
+
+static void xsane_back_gtk_decision_ok_callback(GtkWidget *widget, gpointer data)
+{
+ gint *decision_flag = (gint *) data;
+
+  DBG(DBG_proc, "xsane_back_gtk_decision_ok_callback\n");
+
+  gtk_widget_destroy(widget->parent->parent->parent->parent);
+  xsane.back_gtk_message_dialog_active--;
+
+  if (decision_flag)
+  {
+    *decision_flag = 1;
+  }
+}
+
+/* ----------------------------------------------------------------------------------------------------------------- */
+
+static void xsane_back_gtk_decision_reject_callback(GtkWidget *widget, gpointer data)
+{
+ gint *decision_flag = (gint *) data;
+
+  DBG(DBG_proc, "xsane_back_gtk_decision_reject_callback\n");
+
+  gtk_widget_destroy(widget->parent->parent->parent->parent);
+  xsane.back_gtk_message_dialog_active--;
+
+  if (decision_flag)
+  {
+    *decision_flag = -1;
+  }
 }
 
 /* ----------------------------------------------------------------------------------------------------------------- */
 
 gint xsane_back_gtk_decision(gchar *title, gchar **xpm_d,  gchar *message, gchar *oktext, gchar *rejecttext, int wait)
 {
- GtkWidget *main_vbox, *hbox, *label, *button;
+ GtkWidget *main_vbox, *hbox, *label, *button, *frame;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  GtkWidget *pixmapwidget;
+ GtkWidget *decision_dialog;
+ GtkAccelGroup *accelerator_group;
+ gint decision_flag;
+ gint *decision_flag_ptr = NULL;
 
   DBG(DBG_proc, "xsane_back_gtk_decision\n");
 
-  if (xsane.back_gtk_message_dialog_active)
+  if (wait)
   {
-    DBG(DBG_error0, "%s: %s\n", title, message);
-    return TRUE;
+    decision_flag_ptr = &decision_flag;
   }
-  xsane.back_gtk_message_dialog_active = 1;
+
+  xsane.back_gtk_message_dialog_active++;
   decision_dialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_position(GTK_WINDOW(decision_dialog), GTK_WIN_POS_MOUSE);
   gtk_window_set_title(GTK_WINDOW(decision_dialog), title);
-  g_signal_connect(GTK_OBJECT(decision_dialog), "delete_event", GTK_SIGNAL_FUNC(xsane_back_gtk_decision_callback), (void *) -1); /* -1 = cancel */
+  g_signal_connect(GTK_OBJECT(decision_dialog), "delete_event", GTK_SIGNAL_FUNC(xsane_back_gtk_decision_delete_event), (void *) decision_flag_ptr);
 
   xsane_set_window_icon(decision_dialog, 0);
 
+  accelerator_group = gtk_accel_group_new();
+  gtk_window_add_accel_group(GTK_WINDOW(decision_dialog), accelerator_group);
+
+  /* create a frame */
+  frame = gtk_frame_new(NULL);
+  gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
+  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
+  gtk_container_add(GTK_CONTAINER(decision_dialog), frame);
+  gtk_widget_show(frame);
+
   /* create the main vbox */
-  main_vbox = gtk_vbox_new(TRUE, 5);
+  main_vbox = gtk_vbox_new(FALSE, 5);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);
   gtk_widget_show(main_vbox);
+  gtk_container_add(GTK_CONTAINER(frame), main_vbox);
 
-  gtk_container_add(GTK_CONTAINER(decision_dialog), main_vbox);
-
+  /* create a horizontal box to put the icon and the text insode */
   hbox = gtk_hbox_new(FALSE, 2);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
   gtk_box_pack_start(GTK_BOX(main_vbox), hbox, FALSE, FALSE, 0);
@@ -575,7 +706,7 @@ gint xsane_back_gtk_decision(gchar *title, gchar **xpm_d,  gchar *message, gchar
   /* the confirmation button */
   button = gtk_button_new_with_label(oktext);
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-  g_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) xsane_back_gtk_decision_callback, (void *) 1 /* confirm */);
+  g_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) xsane_back_gtk_decision_ok_callback, (void *) decision_flag_ptr);
   gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 5);
   gtk_widget_grab_default(button);
   gtk_widget_show(button);
@@ -584,14 +715,19 @@ gint xsane_back_gtk_decision(gchar *title, gchar **xpm_d,  gchar *message, gchar
   if (rejecttext) /* the rejection button */
   {
     button = gtk_button_new_with_label(rejecttext);
-    g_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) xsane_back_gtk_decision_callback, (void *) -1 /* reject */);
+    g_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) xsane_back_gtk_decision_reject_callback, (void *) decision_flag_ptr);
     gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 5);
     gtk_widget_show(button);
   }
-  gtk_widget_show(hbox);
 
+  /* if rejectbutton is available then the following command is valid for the reject button */
+  /* otherwise it is valid for the ok button */
+  gtk_widget_add_accelerator(button, "clicked", accelerator_group, GDK_Escape, 0, DEF_GTK_ACCEL_LOCKED);
+
+  gtk_widget_show(hbox);
   gtk_widget_show(decision_dialog);
 
+
   while (gtk_events_pending())
   {
     gtk_main_iteration();
@@ -624,11 +760,26 @@ gint xsane_back_gtk_decision(gchar *title, gchar **xpm_d,  gchar *message, gchar
 
 /* ----------------------------------------------------------------------------------------------------------------- */
 
+void xsane_back_gtk_ipc_dialog_callback(gpointer data, gint source, GdkInputCondition cond)
+{
+ char message[256];
+ size_t bytes;
+
+  DBG(DBG_proc, "xsane_back_gtk_message\n");
+
+  bytes = read(xsane.ipc_pipefd[0], message, 255);
+  message[bytes] = 0;
+
+  xsane_back_gtk_decision(ERR_HEADER_CHILD_PROCESS_ERROR, (gchar **) error_xpm, message, BUTTON_CLOSE, 0 /* no reject text */, FALSE);
+}
+
+/* ----------------------------------------------------------------------------------------------------------------- */
+
 void xsane_back_gtk_message(gchar *title, gchar **icon_xpm, gchar *message, int wait)
 {
   DBG(DBG_proc, "xsane_back_gtk_message\n");
 
-  xsane_back_gtk_decision(title, icon_xpm, message, BUTTON_OK, 0 /* no reject text */, wait);
+  xsane_back_gtk_decision(title, icon_xpm, message, BUTTON_CLOSE, 0 /* no reject text */, wait);
 }
 
 /* ----------------------------------------------------------------------------------------------------------------- */
@@ -691,6 +842,231 @@ void xsane_back_gtk_info(gchar *info, int wait)
   }
 }
 
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
+GtkWidget *fileselection;
+char *fileselection_filetype = NULL;
+
+static void xsane_back_gtk_filetype_callback(GtkWidget *widget, gpointer data)
+{
+ char *extension, *filename;
+ char buffer[256];
+ char *new_filetype = (char *) data;
+
+  DBG(DBG_proc, "xsane_filetype_callback\n");
+
+  filename = strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselection)));
+
+  if ((new_filetype) && (*new_filetype))
+  {
+    extension = strrchr(filename, '.');
+
+    if ((extension) && (extension != filename))
+    {
+      if ( (!strcasecmp(extension, ".pnm"))  || (!strcasecmp(extension, ".raw"))
+        || (!strcasecmp(extension, ".png"))  || (!strcasecmp(extension, ".ps"))
+        || (!strcasecmp(extension, ".rgba"))
+        || (!strcasecmp(extension, ".tiff")) || (!strcasecmp(extension, ".tif"))
+        || (!strcasecmp(extension, ".text")) || (!strcasecmp(extension, ".txt"))
+        || (!strcasecmp(extension, ".jpg"))  || (!strcasecmp(extension, ".jpeg"))
+         ) /* remove filetype extension */
+      {
+        *extension = 0; /* remove extension */
+      }
+    }
+    snprintf(buffer, sizeof(buffer), "%s%s", filename, new_filetype);
+    free(filename);
+    filename = strdup(buffer);
+  }
+
+  if (fileselection_filetype)
+  {
+    free(fileselection_filetype);
+    fileselection_filetype = NULL;
+  }
+
+  if (data)
+  {
+    fileselection_filetype = strdup(new_filetype); 
+  }
+
+  gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection), filename);
+
+  free(filename);
+}
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
+void xsane_back_gtk_filetype_menu_set_history(GtkWidget *xsane_filetype_option_menu, char *filetype)
+{
+ int filetype_nr;
+ int select_item;
+
+  filetype_nr = 0;
+  select_item = 0;
+
+#ifdef HAVE_LIBJPEG
+  filetype_nr++;
+  if ( (filetype) && (!strcasecmp(filetype, XSANE_FILETYPE_JPEG)) )
+  {
+    select_item = filetype_nr;
+  }
+#endif
+
+#ifdef HAVE_LIBPNG
+#ifdef HAVE_LIBZ
+  filetype_nr++;
+  if ( (filetype) && (!strcasecmp(filetype, XSANE_FILETYPE_PNG)) )
+  {
+    select_item = filetype_nr;
+  }
+#endif
+#endif
+
+  filetype_nr++;
+  if ( (filetype) && (!strcasecmp(filetype, XSANE_FILETYPE_PNM)) )
+  {
+    select_item = filetype_nr;
+  }
+
+  filetype_nr++;
+  if ( (filetype) && (!strcasecmp(filetype, XSANE_FILETYPE_PS)) )
+  {
+    select_item = filetype_nr;
+  }
+
+#ifdef SUPPORT_RGBA
+  filetype_nr++;
+  if ( (filetype) && (!strcasecmp(filetype, XSANE_FILETYPE_RGBA)) )
+  {
+    select_item = filetype_nr;
+  }
+#endif
+
+  filetype_nr++;
+  if ( (filetype) && (!strcasecmp(filetype, XSANE_FILETYPE_TEXT)) )
+  {
+    select_item = filetype_nr;
+  }
+
+#ifdef HAVE_LIBTIFF
+  filetype_nr++;
+  if ( (filetype) && (!strcasecmp(filetype, XSANE_FILETYPE_TIFF)) )
+  {
+    select_item = filetype_nr;
+  }
+#endif
+
+  gtk_option_menu_set_history(GTK_OPTION_MENU(xsane_filetype_option_menu), select_item);
+}
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
+GtkWidget *xsane_back_gtk_filetype_menu_new(char *filetype, GtkSignalFunc filetype_callback)
+{
+ GtkWidget *xsane_filetype_menu, *xsane_filetype_item;
+ GtkWidget *xsane_filetype_option_menu;
+ int filetype_nr;
+ int select_item;
+
+  xsane_filetype_menu = gtk_menu_new();
+
+  xsane_filetype_item = gtk_menu_item_new_with_label(MENU_ITEM_FILETYPE_BY_EXT);
+  gtk_container_add(GTK_CONTAINER(xsane_filetype_menu), xsane_filetype_item);
+  g_signal_connect(GTK_OBJECT(xsane_filetype_item), "activate", filetype_callback, (void *) XSANE_FILETYPE_BY_EXT);
+  gtk_widget_show(xsane_filetype_item);
+
+  filetype_nr = 0;
+  select_item = 0;
+
+#ifdef HAVE_LIBJPEG
+  xsane_filetype_item = gtk_menu_item_new_with_label(MENU_ITEM_FILETYPE_JPEG);
+  gtk_container_add(GTK_CONTAINER(xsane_filetype_menu), xsane_filetype_item);
+  g_signal_connect(GTK_OBJECT(xsane_filetype_item), "activate", filetype_callback, (void *) XSANE_FILETYPE_JPEG);
+  gtk_widget_show(xsane_filetype_item);
+  filetype_nr++;
+  if ( (filetype) && (!strcasecmp(filetype, XSANE_FILETYPE_JPEG)) )
+  {
+    select_item = filetype_nr;
+  }
+#endif
+
+#ifdef HAVE_LIBPNG
+#ifdef HAVE_LIBZ
+  xsane_filetype_item = gtk_menu_item_new_with_label(MENU_ITEM_FILETYPE_PNG);
+  gtk_container_add(GTK_CONTAINER(xsane_filetype_menu), xsane_filetype_item);
+  g_signal_connect(GTK_OBJECT(xsane_filetype_item), "activate", filetype_callback, (void *) XSANE_FILETYPE_PNG);
+  gtk_widget_show(xsane_filetype_item);
+  filetype_nr++;
+  if ( (filetype) && (!strcasecmp(filetype, XSANE_FILETYPE_PNG)) )
+  {
+    select_item = filetype_nr;
+  }
+#endif
+#endif
+
+  xsane_filetype_item = gtk_menu_item_new_with_label(MENU_ITEM_FILETYPE_PNM);
+  gtk_container_add(GTK_CONTAINER(xsane_filetype_menu), xsane_filetype_item);
+  g_signal_connect(GTK_OBJECT(xsane_filetype_item), "activate", filetype_callback, (void *) XSANE_FILETYPE_PNM);
+  gtk_widget_show(xsane_filetype_item);
+  filetype_nr++;
+  if ( (filetype) && (!strcasecmp(filetype, XSANE_FILETYPE_PNM)) )
+  {
+    select_item = filetype_nr;
+  }
+
+  xsane_filetype_item = gtk_menu_item_new_with_label(MENU_ITEM_FILETYPE_PS);
+  gtk_container_add(GTK_CONTAINER(xsane_filetype_menu), xsane_filetype_item);
+  g_signal_connect(GTK_OBJECT(xsane_filetype_item), "activate", filetype_callback, (void *) XSANE_FILETYPE_PS);
+  gtk_widget_show(xsane_filetype_item);
+  filetype_nr++;
+  if ( (filetype) && (!strcasecmp(filetype, XSANE_FILETYPE_PS)) )
+  {
+    select_item = filetype_nr;
+  }
+
+#ifdef SUPPORT_RGBA
+  xsane_filetype_item = gtk_menu_item_new_with_label(MENU_ITEM_FILETYPE_RGBA);
+  gtk_container_add(GTK_CONTAINER(xsane_filetype_menu), xsane_filetype_item);
+  g_signal_connect(GTK_OBJECT(xsane_filetype_item), "activate", filetype_callback, (void *) XSANE_FILETYPE_RGBA);
+  gtk_widget_show(xsane_filetype_item);
+  filetype_nr++;
+  if ( (filetype) && (!strcasecmp(filetype, XSANE_FILETYPE_RGBA)) )
+  {
+    select_item = filetype_nr;
+  }
+#endif
+
+  xsane_filetype_item = gtk_menu_item_new_with_label(MENU_ITEM_FILETYPE_TEXT);
+  gtk_container_add(GTK_CONTAINER(xsane_filetype_menu), xsane_filetype_item);
+  g_signal_connect(GTK_OBJECT(xsane_filetype_item), "activate", filetype_callback, (void *) XSANE_FILETYPE_TEXT);
+  gtk_widget_show(xsane_filetype_item);
+  filetype_nr++;
+  if ( (filetype) && (!strcasecmp(filetype, XSANE_FILETYPE_TEXT)) )
+  {
+    select_item = filetype_nr;
+  }
+
+#ifdef HAVE_LIBTIFF
+  xsane_filetype_item = gtk_menu_item_new_with_label(MENU_ITEM_FILETYPE_TIFF);
+  gtk_container_add(GTK_CONTAINER(xsane_filetype_menu), xsane_filetype_item);
+  g_signal_connect(GTK_OBJECT(xsane_filetype_item), "activate", filetype_callback, (void *) XSANE_FILETYPE_TIFF);
+  gtk_widget_show(xsane_filetype_item);
+  filetype_nr++;
+  if ( (filetype) && (!strcasecmp(filetype, XSANE_FILETYPE_TIFF)) )
+  {
+    select_item = filetype_nr;
+  }
+#endif
+
+  xsane_filetype_option_menu = gtk_option_menu_new();
+  xsane_back_gtk_set_tooltip(xsane.tooltips, xsane_filetype_option_menu, DESC_FILETYPE);
+  gtk_option_menu_set_menu(GTK_OPTION_MENU(xsane_filetype_option_menu), xsane_filetype_menu);
+  gtk_option_menu_set_history(GTK_OPTION_MENU(xsane_filetype_option_menu), select_item);
+
+ return (xsane_filetype_option_menu);
+}
+
 /* ----------------------------------------------------------------------------------------------------------------- */
 
 static void xsane_back_gtk_get_filename_button_clicked(GtkWidget *w, gpointer data)
@@ -703,15 +1079,20 @@ static void xsane_back_gtk_get_filename_button_clicked(GtkWidget *w, gpointer da
 
 /* ----------------------------------------------------------------------------------------------------------------- */
 
-int xsane_back_gtk_get_filename(const char *label, const char *default_name, size_t max_len, char *filename,
-                                int show_fileopts, int shorten_path, int hide_file_list)
+int xsane_back_gtk_get_filename(const char *label, const char *default_name, size_t max_len, char *filename, char **filetype,
+                                int show_fileopts, int shorten_path, int select_directory, int show_filetype_menu)
 {
  int cancel = 0, ok = 0, destroy = 0;
- GtkWidget *fileselection;
  GtkAccelGroup *accelerator_group;
+ GtkWidget *xsane_filetype_option_menu;
 
   DBG(DBG_proc, "xsane_back_gtk_get_filename\n");
 
+  if (fileselection)
+  {
+    gdk_beep();
+    return -1; /* cancel => do not allow to open more than one fileselection dialog */
+  }
 
   fileselection = gtk_file_selection_new((char *) label);
   accelerator_group = gtk_accel_group_new();
@@ -724,19 +1105,32 @@ int xsane_back_gtk_get_filename(const char *label, const char *default_name, siz
                               accelerator_group, GDK_Escape, 0, DEF_GTK_ACCEL_LOCKED);
 
   g_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fileselection)->ok_button), "clicked", (GtkSignalFunc) xsane_back_gtk_get_filename_button_clicked, &ok);
-  if (default_name)
+
+  if (select_directory)
+  {
+    DBG(DBG_info, "xsane_back_gtk_get_filename: select directory\n");
+    gtk_widget_hide(GTK_FILE_SELECTION(fileselection)->file_list->parent);
+    gtk_widget_hide(GTK_FILE_SELECTION(fileselection)->fileop_del_file);
+    gtk_widget_hide(GTK_FILE_SELECTION(fileselection)->fileop_ren_file);
+    gtk_widget_hide(GTK_FILE_SELECTION(fileselection)->selection_entry);
+
+    gtk_widget_set_size_request(GTK_FILE_SELECTION(fileselection)->dir_list, 280, 230);
+
+    if (default_name) /* add "/." to end of directory name so that the gtkfilesel* behaves correct */
+    {
+     char directory_name[PATH_MAX];
+
+      snprintf(directory_name, sizeof(directory_name), "%s/.", default_name);
+      DBG(DBG_info, "xsane_back_gtk_get_filename: directory_name =%s\n", directory_name);
+      gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection), (char *) directory_name);
+    }
+  }
+  else if (default_name) /* select file */
   {
     DBG(DBG_info, "xsane_back_gtk_get_filename: default_name =%s\n", default_name);
     gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection), (char *) default_name);
   }
 
-  if (hide_file_list)
-  {
-    DBG(DBG_info, "xsane_back_gtk_get_filename: hiding file-list and delete-file-widget\n");
-    gtk_widget_hide(GTK_FILE_SELECTION(fileselection)->file_list->parent);
-    gtk_widget_hide(GTK_FILE_SELECTION(fileselection)->fileop_del_file);
-  }
-
   if (show_fileopts)
   {
     DBG(DBG_info, "xsane_back_gtk_get_filename: showing file-options\n");
@@ -748,6 +1142,45 @@ int xsane_back_gtk_get_filename(const char *label, const char *default_name, siz
     gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(fileselection));
   }
 
+  if (show_filetype_menu)
+  {
+   GtkWidget *hbox;
+   GtkWidget *vbox;
+   GtkWidget *label;
+
+    DBG(DBG_info, "xsane_back_gtk_get_filename: showing filetype menu\n");
+
+    if (fileselection_filetype)
+    {
+      free(fileselection_filetype);
+    }
+
+    if ((filetype) && (*filetype))
+    {
+      fileselection_filetype = strdup(*filetype);
+    }
+    else
+    {
+      fileselection_filetype = NULL;
+    }
+
+    vbox = gtk_vbox_new(FALSE, 2);
+    gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(fileselection)->action_area), vbox, TRUE, TRUE, 0);
+    gtk_widget_show(vbox);
+
+    hbox = gtk_hbox_new(FALSE, 2);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    gtk_widget_show(hbox);
+
+    label = gtk_label_new("Filetype"); // XXXXXXXXXXX
+    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+    gtk_widget_show(label);
+
+    xsane_filetype_option_menu = xsane_back_gtk_filetype_menu_new(fileselection_filetype, (GtkSignalFunc) xsane_back_gtk_filetype_callback);
+    gtk_box_pack_start(GTK_BOX(hbox), xsane_filetype_option_menu, TRUE, TRUE, 2);
+    gtk_widget_show(xsane_filetype_option_menu);
+  }
+
   gtk_widget_show(fileselection);
 
   DBG(DBG_info, "xsane_back_gtk_get_filename: waiting for user action\n");
@@ -766,7 +1199,33 @@ int xsane_back_gtk_get_filename(const char *label, const char *default_name, siz
 
     DBG(DBG_info, "ok button pressed\n");
 
+    if ((filetype) && (*filetype))
+    {
+      free(*filetype);
+      *filetype = NULL;
+    }
+
+    if (fileselection_filetype)
+    {
+      if (filetype)
+      {
+        *filetype = strdup(fileselection_filetype);
+      }
+
+      free(fileselection_filetype);
+      fileselection_filetype = NULL;
+    }
+
     strncpy(filename, gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselection)), max_len - 1);
+
+#ifndef HAVE_GTK2
+    /* in gtk1 we have to remove the text that is defined in the selection entry to get a proper behaviour */
+    if (select_directory)
+    {
+      *(filename+strlen(filename)-strlen(gtk_entry_get_text(GTK_ENTRY(GTK_FILE_SELECTION(fileselection)->selection_entry)))) = '\0';
+    }
+#endif
+
     filename[max_len - 1] = '\0';
 
     len = strlen(filename);
@@ -790,6 +1249,8 @@ int xsane_back_gtk_get_filename(const char *label, const char *default_name, siz
     gtk_widget_destroy(fileselection);
   }
 
+  fileselection = NULL;
+
   return ok ? 0 : -1;
 }
 
@@ -989,6 +1450,11 @@ void xsane_back_gtk_range_new(GtkWidget * parent, const char *name, gfloat val,
     digits = (int) (log10(1/quant)+0.8); /* set number of digits in dependacne of quantization */
   }
 
+  if (digits < 0)
+  {
+    digits = 0;
+  }
+
   hbox = gtk_hbox_new(FALSE, 2);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 2);
   gtk_box_pack_start(GTK_BOX(parent), hbox, FALSE, FALSE, 0);
@@ -996,8 +1462,8 @@ void xsane_back_gtk_range_new(GtkWidget * parent, const char *name, gfloat val,
   label = gtk_label_new((char *) name);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
 
-  elem->data = gtk_adjustment_new(val, min, max, quant, quant*10, (max-min) * 1e-40);
-  /* 1e-40 => hscrollbar has an unwanted side effect: the maximum is not the maximum */
+  elem->data = gtk_adjustment_new(val, min, max, quant, quant*10, (max-min) * 1e-30);
+  /* 1e-30 => hscrollbar has an unwanted side effect: the maximum is not the maximum */
   /* of the given range, it is reduced by the page_size, so it has to be very small */
 
   /* value label */
@@ -1021,6 +1487,12 @@ void xsane_back_gtk_range_new(GtkWidget * parent, const char *name, gfloat val,
   /* spinbutton */
   if (preferences.show_range_mode & 4)
   {
+#ifndef HAVE_GTK2
+    if (digits > 5)
+    {
+      digits = 5;
+    }
+#endif
     spinbutton = gtk_spin_button_new(GTK_ADJUSTMENT(elem->data), 0, digits);
 
     if (preferences.show_range_mode & 3) /* slider also visible */
@@ -1204,7 +1676,7 @@ static void xsane_back_gtk_text_entry_callback(GtkWidget *w, gpointer data)
 {
  GSGDialogElement *elem = data;
  const SANE_Option_Descriptor *opt;
- gchar *text;
+ const gchar *text;
  int opt_num;
  char *buf;
 
@@ -1303,10 +1775,20 @@ static void xsane_back_gtk_panel_destroy(void)
 
   DBG(DBG_proc, "xsane_back_gtk_panel_destroy\n");
 
+  if (!xsane.xsane_hbox)
+  {
+    DBG(DBG_proc, "xsane_back_gtk_panel_destroy: panel does not exist\n");
+   return;
+  }
+
   gtk_widget_destroy(xsane.xsane_hbox);
   gtk_widget_destroy(xsane.standard_hbox);
   gtk_widget_destroy(xsane.advanced_hbox);
 
+  xsane.xsane_hbox    = NULL;
+  xsane.standard_hbox = NULL;
+  xsane.advanced_hbox = NULL;
+
   /* free the menu labels of integer/fix-point word-lists: */
   for (i = 0; i < xsane.num_elements; ++i)
   {
@@ -1605,13 +2087,23 @@ void xsane_set_sensitivity(SANE_Int sensitivity)
 
   if (xsane.shell)
   {
+    /* clear or rebuild histogram */
+    if (sensitivity)
+    {
+      xsane_update_histogram(TRUE /* update raw */);
+    }
+    else
+    {
+      xsane_clear_histogram(&xsane.histogram_raw);
+      xsane_clear_histogram(&xsane.histogram_enh);
+    }
+
     gtk_widget_set_sensitive(xsane.menubar, sensitivity); 
     gtk_widget_set_sensitive(xsane.xsane_window, sensitivity); 
     gtk_widget_set_sensitive(GTK_WIDGET(xsane.start_button), sensitivity);
     gtk_widget_set_sensitive(xsane.standard_options_shell, sensitivity);
     gtk_widget_set_sensitive(xsane.advanced_options_shell, sensitivity);
     gtk_widget_set_sensitive(xsane.histogram_dialog, sensitivity);
-
 #ifdef HAVE_WORKING_GTK_GAMMACURVE
     gtk_widget_set_sensitive(xsane.gamma_dialog, sensitivity);
 #endif
@@ -1631,6 +2123,12 @@ void xsane_set_sensitivity(SANE_Int sensitivity)
     gtk_widget_set_sensitive(xsane.fax_dialog, sensitivity);
   }
 
+  if (xsane.batch_scan_dialog)
+  {
+    gtk_widget_set_sensitive(xsane.batch_scan_button_box, sensitivity);
+    gtk_widget_set_sensitive(xsane.batch_scan_action_box, sensitivity);
+  }
+
 #if 0
   xsane_back_gtk_set_sensitivity(sensitivity);
 #endif
