@@ -77,13 +77,13 @@ void xsane_range_new_with_pixmap(GdkWindow *window, GtkBox *parent, const char *
 static void xsane_outputfilename_changed_callback(GtkWidget *widget, gpointer data);
 void xsane_set_filename(gchar *filename);
 void xsane_separator_new(GtkWidget *xsane_parent, int dist);
+void xsane_vseparator_new(GtkWidget *xsane_parent, int dist);
 GtkWidget *xsane_info_table_text_new(GtkWidget *table, gchar *text, int row, int colomn);
 GtkWidget *xsane_info_text_new(GtkWidget *parent, gchar *text);
 void xsane_refresh_dialog(void);
-void xsane_set_sensitivity(SANE_Int sensitivity);
 void xsane_update_param(void *arg);
 void xsane_define_output_filename(void);
-int xsane_identify_output_format(char *filename, char **ext);
+int xsane_identify_output_format(char *filename, char *filetype, char **ext);
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
@@ -405,45 +405,47 @@ void xsane_define_maximum_output_size()
       case XSANE_SAVE:
 
         xsane_define_output_filename();
-        xsane.xsane_output_format = xsane_identify_output_format(xsane.output_filename, 0);
+        xsane.xsane_output_format = xsane_identify_output_format(xsane.output_filename, preferences.filetype, 0);
 
-        preview_set_maximum_output_size(xsane.preview, INF, INF);
+        preview_set_maximum_output_size(xsane.preview, INF, INF, 0);
        break;
 
       case XSANE_VIEWER:
-        preview_set_maximum_output_size(xsane.preview, INF, INF);
+        preview_set_maximum_output_size(xsane.preview, INF, INF, 0);
        break;
 
       case XSANE_COPY:
-        if (preferences.psrotate) /* rotate: landscape */
+        if (preferences.paper_orientation >= 8) /* rotate: landscape */
         {
           preview_set_maximum_output_size(xsane.preview,
                                           preferences.printer[preferences.printernr]->height / xsane.zoom_y,
-                                          preferences.printer[preferences.printernr]->width  / xsane.zoom_x);
+                                          preferences.printer[preferences.printernr]->width  / xsane.zoom_x,
+                                          preferences.paper_orientation);
         }
         else /* do not rotate: portrait */
         {
           preview_set_maximum_output_size(xsane.preview,
                                           preferences.printer[preferences.printernr]->width  / xsane.zoom_x,
-                                          preferences.printer[preferences.printernr]->height / xsane.zoom_y);
+                                          preferences.printer[preferences.printernr]->height / xsane.zoom_y,
+                                          preferences.paper_orientation);
         }
        break;
 
       case XSANE_FAX:
-        preview_set_maximum_output_size(xsane.preview, preferences.fax_width, preferences.fax_height);
+        preview_set_maximum_output_size(xsane.preview, preferences.fax_width, preferences.fax_height, 0);
        break;
 
       case XSANE_MAIL:
-        preview_set_maximum_output_size(xsane.preview, INF, INF);
+        preview_set_maximum_output_size(xsane.preview, INF, INF, 0);
        break;
 
       default:
-        preview_set_maximum_output_size(xsane.preview, INF, INF);
+        preview_set_maximum_output_size(xsane.preview, INF, INF, 0);
     }
   }
   else
   {
-    preview_set_maximum_output_size(xsane.preview, INF, INF);
+    preview_set_maximum_output_size(xsane.preview, INF, INF, 0);
   }
 }
 
@@ -478,7 +480,7 @@ gint xsane_authorization_callback(SANE_String_Const resource,
  GtkWidget *authorize_dialog, *vbox, *hbox, *button, *label;
  GtkWidget *username_widget, *password_widget;
  char buf[SANE_MAX_PASSWORD_LEN+SANE_MAX_USERNAME_LEN+128];
- char *input;
+ const gchar *input;
  char *resource_string;
  int len;
  int resource_len;
@@ -1063,8 +1065,8 @@ void xsane_range_new(GtkBox *parent, char *labeltext, const char *desc,
   label = gtk_label_new(labeltext);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 1);
 
-  *data = gtk_adjustment_new(*val, min, max, quant, page_step, (max-min) * 1e-40);
-  /* 1e-40 => hscrollbar has an unwanted side effect: the maximum is not the maximum */
+  *data = gtk_adjustment_new(*val, min, max, quant, page_step, (max-min) * 1e-30);
+  /* 1e-30 => hscrollbar has an unwanted side effect: the maximum is not the maximum */
   /* of the given range, it is reduced by the page_size, so it has to be very small */
 
   /* value label */
@@ -1156,8 +1158,8 @@ void xsane_range_new_with_pixmap(GdkWindow *window, GtkBox *parent, const char *
   gtk_widget_show(pixmapwidget);
   gdk_drawable_unref(pixmap);
 
-  *data = gtk_adjustment_new(*val, min, max, quant, page_step, (max-min) * 1e-40);
-  /* 1e-40 => hscrollbar has an unwanted side effect: the maximum is not the maximum */
+  *data = gtk_adjustment_new(*val, min, max, quant, page_step, (max-min) * 1e-30);
+  /* 1e-30 => hscrollbar has an unwanted side effect: the maximum is not the maximum */
   /* of the given range, it is reduced by the page_size, so it has to be very small */
 
   /* value label */
@@ -1243,17 +1245,6 @@ static void xsane_browse_filename_callback(GtkWidget *widget, gpointer data)
 
   xsane_set_sensitivity(FALSE);
 
-  if (xsane.filetype) /* set filetype to "by ext." */
-  {
-   char buffer[256];
-
-    snprintf(buffer, sizeof(buffer), "%s%s", preferences.filename, xsane.filetype);
-    free(preferences.filename);
-    free(xsane.filetype);
-    xsane.filetype = NULL;
-    preferences.filename = strdup(buffer);
-  }
-
   if (preferences.filename) /* make sure a correct filename is defined */
   {
     strncpy(filename, preferences.filename, sizeof(filename));
@@ -1267,7 +1258,7 @@ static void xsane_browse_filename_callback(GtkWidget *widget, gpointer data)
   snprintf(windowname, sizeof(windowname), "%s %s %s", xsane.prog_name, WINDOW_OUTPUT_FILENAME, xsane.device_text);
 
   umask((mode_t) preferences.directory_umask); /* define new file permissions */    
-  xsane_back_gtk_get_filename(windowname, filename, sizeof(filename), filename, TRUE, TRUE, FALSE);
+  xsane_back_gtk_get_filename(windowname, filename, sizeof(filename), filename, &preferences.filetype, TRUE, TRUE, FALSE, TRUE);
   umask(XSANE_DEFAULT_UMASK); /* define new file permissions */    
 
   if (preferences.filename)
@@ -1275,16 +1266,17 @@ static void xsane_browse_filename_callback(GtkWidget *widget, gpointer data)
     free((void *) preferences.filename);
   }
 
+  preferences.filename = strdup(filename);
+
   xsane_set_sensitivity(TRUE);
 
-  preferences.filename = strdup(filename);
+  xsane_back_gtk_filetype_menu_set_history(xsane.filetype_option_menu, preferences.filetype);
 
   /* correct length of filename counter if it is shorter than minimum length */
   xsane_update_counter_in_filename(&preferences.filename, FALSE, 0, preferences.filename_counter_len);
 
   xsane_set_filename(preferences.filename);
 
-  gtk_option_menu_set_history(GTK_OPTION_MENU(xsane.filetype_option_menu), 0); /* set menu to "by ext" */
   xsane_define_maximum_output_size(); /* is necessary in postscript mode */
 }
 
@@ -1294,6 +1286,8 @@ void xsane_set_filename(gchar *filename)
 {
   g_signal_handlers_block_by_func(GTK_OBJECT(xsane.outputfilename_entry), (GtkSignalFunc) xsane_outputfilename_changed_callback, NULL);
   gtk_entry_set_text(GTK_ENTRY(xsane.outputfilename_entry), (char *) filename); /* update filename in entry */
+  gtk_entry_set_position(GTK_ENTRY(xsane.outputfilename_entry), strlen(filename)); /* set cursor to right position of filename */
+
   g_signal_handlers_unblock_by_func(GTK_OBJECT(xsane.outputfilename_entry), (GtkSignalFunc) xsane_outputfilename_changed_callback, NULL);
 }
 
@@ -1310,12 +1304,18 @@ static void xsane_filename_counter_step_callback(GtkWidget *widget, gpointer dat
 
 static void xsane_filetype_callback(GtkWidget *widget, gpointer data)
 {
+ char *new_filetype = (char *) data;
+ char buffer[PATH_MAX];
+ char *filename;
+
   DBG(DBG_proc, "xsane_filetype_callback\n");
 
-  if (data)
+  filename = preferences.filename;
+
+  if ((new_filetype) && (*new_filetype)) /* filetype exists and is not empty (by ext) */
   {
-   char *extension, *filename;
- 
+   char *extension;
+
     extension = strrchr(preferences.filename, '.');
 
     if ((extension) && (extension != preferences.filename))
@@ -1324,30 +1324,27 @@ static void xsane_filetype_callback(GtkWidget *widget, gpointer data)
         || (!strcasecmp(extension, ".png"))  || (!strcasecmp(extension, ".ps"))
         || (!strcasecmp(extension, ".rgba"))
         || (!strcasecmp(extension, ".tiff")) || (!strcasecmp(extension, ".tif"))
+        || (!strcasecmp(extension, ".text")) || (!strcasecmp(extension, ".txt"))
         || (!strcasecmp(extension, ".jpg"))  || (!strcasecmp(extension, ".jpeg"))
          ) /* remove filetype extension */
       {
-        filename = preferences.filename;
         *extension = 0; /* remove extension */
-        preferences.filename = strdup(filename); /* filename without extension */
-        free(filename); /* free unused memory */
       }
     }
-  }
-  else if (xsane.filetype)
-  {
-   char buffer[256];
-
-    snprintf(buffer, sizeof(buffer), "%s%s", preferences.filename, xsane.filetype);
-    free(preferences.filename);
-    free(xsane.filetype);
-    xsane.filetype = NULL;
+    snprintf(buffer, sizeof(buffer), "%s%s", filename, new_filetype);
+    free(filename);
     preferences.filename = strdup(buffer);
   }
 
-  if (data)
+  if (preferences.filetype)
   {
-    xsane.filetype = strdup((char *) data); /* set extension for filename */
+    free(preferences.filetype);
+    preferences.filetype = NULL;
+  }
+
+  if (new_filetype)
+  {
+    preferences.filetype = strdup(new_filetype);
   }
 
   /* correct length of filename counter if it is shorter than minimum length */
@@ -1378,14 +1375,12 @@ void xsane_outputfilename_new(GtkWidget *vbox)
  GtkWidget *hbox;
  GtkWidget *text;
  GtkWidget *button;
- GtkWidget *xsane_filetype_menu, *xsane_filetype_item;
  GtkWidget *xsane_filename_counter_step_option_menu;
  GtkWidget *xsane_filename_counter_step_menu;
  GtkWidget *xsane_filename_counter_step_item;
  GtkWidget *xsane_label;
  gchar buf[200];
  int i,j;
- int filetype_nr;
  int select_item = 0;
 
   DBG(DBG_proc, "xsane_outputfilename_new\n");
@@ -1405,6 +1400,8 @@ void xsane_outputfilename_new(GtkWidget *vbox)
   xsane_back_gtk_set_tooltip(xsane.tooltips, text, DESC_FILENAME);
   gtk_entry_set_max_length(GTK_ENTRY(text), 255);
   gtk_entry_set_text(GTK_ENTRY(text), (char *) preferences.filename);
+  gtk_entry_set_position(GTK_ENTRY(text), strlen(preferences.filename)); /* set cursor to right position of filename */
+
   gtk_box_pack_start(GTK_BOX(hbox), text, TRUE, TRUE, 4);
   g_signal_connect(GTK_OBJECT(text), "changed", (GtkSignalFunc) xsane_outputfilename_changed_callback, NULL);
 
@@ -1452,102 +1449,8 @@ void xsane_outputfilename_new(GtkWidget *vbox)
   gtk_option_menu_set_menu(GTK_OPTION_MENU(xsane_filename_counter_step_option_menu), xsane_filename_counter_step_menu);
   gtk_option_menu_set_history(GTK_OPTION_MENU(xsane_filename_counter_step_option_menu), select_item);
 
-  /* filetype */
-
-  xsane_filetype_menu = gtk_menu_new();
-
-  xsane_filetype_item = gtk_menu_item_new_with_label(MENU_ITEM_FILETYPE_BY_EXT);
-  gtk_container_add(GTK_CONTAINER(xsane_filetype_menu), xsane_filetype_item);
-  g_signal_connect(GTK_OBJECT(xsane_filetype_item), "activate", (GtkSignalFunc) xsane_filetype_callback, NULL);
-  gtk_widget_show(xsane_filetype_item);
-  filetype_nr = 0;
-  select_item = 0;
-
-#ifdef HAVE_LIBJPEG
-  xsane_filetype_item = gtk_menu_item_new_with_label(MENU_ITEM_FILETYPE_JPEG);
-  gtk_container_add(GTK_CONTAINER(xsane_filetype_menu), xsane_filetype_item);
-  g_signal_connect(GTK_OBJECT(xsane_filetype_item), "activate", (GtkSignalFunc) xsane_filetype_callback, (void *) XSANE_FILETYPE_JPEG);
-  gtk_widget_show(xsane_filetype_item);
-  filetype_nr++;
-  if ( (xsane.filetype) && (!strcasecmp(xsane.filetype, XSANE_FILETYPE_JPEG)) )
-  {
-    select_item = filetype_nr;
-  }
-#endif
-
-#ifdef HAVE_LIBPNG
-#ifdef HAVE_LIBZ
-  xsane_filetype_item = gtk_menu_item_new_with_label(MENU_ITEM_FILETYPE_PNG);
-  gtk_container_add(GTK_CONTAINER(xsane_filetype_menu), xsane_filetype_item);
-  g_signal_connect(GTK_OBJECT(xsane_filetype_item), "activate", (GtkSignalFunc) xsane_filetype_callback, (void *) XSANE_FILETYPE_PNG);
-  gtk_widget_show(xsane_filetype_item);
-  filetype_nr++;
-  if ( (xsane.filetype) && (!strcasecmp(xsane.filetype, XSANE_FILETYPE_PNG)) )
-  {
-    select_item = filetype_nr;
-  }
-#endif
-#endif
-
-  xsane_filetype_item = gtk_menu_item_new_with_label(MENU_ITEM_FILETYPE_PNM);
-  gtk_container_add(GTK_CONTAINER(xsane_filetype_menu), xsane_filetype_item);
-  g_signal_connect(GTK_OBJECT(xsane_filetype_item), "activate", (GtkSignalFunc) xsane_filetype_callback, (void *) XSANE_FILETYPE_PNM);
-  gtk_widget_show(xsane_filetype_item);
-  filetype_nr++;
-  if ( (xsane.filetype) && (!strcasecmp(xsane.filetype, XSANE_FILETYPE_PNM)) )
-  {
-    select_item = filetype_nr;
-  }
-
-  xsane_filetype_item = gtk_menu_item_new_with_label(MENU_ITEM_FILETYPE_PS);
-  gtk_container_add(GTK_CONTAINER(xsane_filetype_menu), xsane_filetype_item);
-  g_signal_connect(GTK_OBJECT(xsane_filetype_item), "activate", (GtkSignalFunc) xsane_filetype_callback, (void *) XSANE_FILETYPE_PS);
-  gtk_widget_show(xsane_filetype_item);
-  filetype_nr++;
-  if ( (xsane.filetype) && (!strcasecmp(xsane.filetype, XSANE_FILETYPE_PS)) )
-  {
-    select_item = filetype_nr;
-  }
-
-  xsane_filetype_item = gtk_menu_item_new_with_label(MENU_ITEM_FILETYPE_RAW);
-  gtk_container_add(GTK_CONTAINER(xsane_filetype_menu), xsane_filetype_item);
-  g_signal_connect(GTK_OBJECT(xsane_filetype_item), "activate", (GtkSignalFunc) xsane_filetype_callback, (void *) XSANE_FILETYPE_RAW);
-  gtk_widget_show(xsane_filetype_item);
-  filetype_nr++;
-  if ( (xsane.filetype) && (!strcasecmp(xsane.filetype, XSANE_FILETYPE_RAW)) )
-  {
-    select_item = filetype_nr;
-  }
-
-#ifdef SUPPORT_RGBA
-  xsane_filetype_item = gtk_menu_item_new_with_label(MENU_ITEM_FILETYPE_RGBA);
-  gtk_container_add(GTK_CONTAINER(xsane_filetype_menu), xsane_filetype_item);
-  g_signal_connect(GTK_OBJECT(xsane_filetype_item), "activate", (GtkSignalFunc) xsane_filetype_callback, (void *) XSANE_FILETYPE_RGBA);
-  gtk_widget_show(xsane_filetype_item);
-  filetype_nr++;
-  if ( (xsane.filetype) && (!strcasecmp(xsane.filetype, XSANE_FILETYPE_RGBA)) )
-  {
-    select_item = filetype_nr;
-  }
-#endif
-
-#ifdef HAVE_LIBTIFF
-  xsane_filetype_item = gtk_menu_item_new_with_label(MENU_ITEM_FILETYPE_TIFF);
-  gtk_container_add(GTK_CONTAINER(xsane_filetype_menu), xsane_filetype_item);
-  g_signal_connect(GTK_OBJECT(xsane_filetype_item), "activate", (GtkSignalFunc) xsane_filetype_callback, (void *) XSANE_FILETYPE_TIFF);
-  gtk_widget_show(xsane_filetype_item);
-  filetype_nr++;
-  if ( (xsane.filetype) && (!strcasecmp(xsane.filetype, XSANE_FILETYPE_TIFF)) )
-  {
-    select_item = filetype_nr;
-  }
-#endif
-
-  xsane.filetype_option_menu = gtk_option_menu_new();
-  xsane_back_gtk_set_tooltip(xsane.tooltips, xsane.filetype_option_menu, DESC_FILETYPE);
+  xsane.filetype_option_menu = xsane_back_gtk_filetype_menu_new(preferences.filetype, (GtkSignalFunc) xsane_filetype_callback);
   gtk_box_pack_end(GTK_BOX(hbox), xsane.filetype_option_menu, FALSE, FALSE, 2);
-  gtk_option_menu_set_menu(GTK_OPTION_MENU(xsane.filetype_option_menu), xsane_filetype_menu);
-  gtk_option_menu_set_history(GTK_OPTION_MENU(xsane.filetype_option_menu), select_item);
   gtk_widget_show(xsane.filetype_option_menu);
 
   xsane_label = gtk_label_new(TEXT_FILETYPE); /* opposite order because of box_pack_end */
@@ -1573,6 +1476,20 @@ void xsane_separator_new(GtkWidget *xsane_parent, int dist)
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
+void xsane_vseparator_new(GtkWidget *xsane_parent, int dist)
+{
+ GtkWidget *xsane_vseparator;
+
+  DBG(DBG_proc, "xsane_vseparator_new\n");
+
+  xsane_vseparator = gtk_vseparator_new();
+  gtk_box_pack_start(GTK_BOX(xsane_parent), xsane_vseparator, FALSE, FALSE, dist);
+  gtk_widget_show(xsane_vseparator);
+
+}
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
 GtkWidget *xsane_info_table_text_new(GtkWidget *table, gchar *text, int row, int colomn)
 {
  GtkWidget *hbox, *label;
@@ -1728,8 +1645,6 @@ void xsane_update_param(void *arg)
 
 void xsane_define_output_filename(void)
 {
- char buffer[256];
-
   DBG(DBG_proc, "xsane_define_output_filename\n");
 
   if (xsane.output_filename)
@@ -1740,52 +1655,44 @@ void xsane_define_output_filename(void)
 
   if (!xsane.force_filename)
   {
-    if (xsane.filetype)
-    {
-      snprintf(buffer, sizeof(buffer), "%s%s", preferences.filename, xsane.filetype);
-      xsane.output_filename = strdup(buffer);
-    }
-    else
-    {
-      xsane.output_filename = strdup(preferences.filename);
-    }
+    xsane.output_filename = strdup(preferences.filename);
   }
   else
   {
-      xsane.output_filename = strdup(xsane.external_filename);
+    xsane.output_filename = strdup(xsane.external_filename);
   }
 }
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
-int xsane_identify_output_format(char *filename, char **ext)
+int xsane_identify_output_format(char *filename, char *filetype, char **ext)
 {
  char *extension;
  int output_format=-1;
 
   DBG(DBG_proc, "xsane_identify_output_format\n");
 
-  extension = strrchr(filename, '.');
-  if (extension)
+  if ((filetype) && (*filetype))
   {
-    extension++; /* skip "." */
+    extension = filetype+1; /* go to filetype, skip leading dot */
+  }
+  else
+  {
+    extension = strrchr(filename, '.');
+    if (extension)
+    {
+      extension++; /* skip "." */
+    }
   }
 
   output_format = XSANE_UNKNOWN;
 
   if (extension)
   {
-    if (!strcasecmp(extension, "raw"))
+    if ( (!strcasecmp(extension, "pnm")) || (!strcasecmp(extension, "ppm")) ||
+         (!strcasecmp(extension, "pgm")) || (!strcasecmp(extension, "pbm")) )
     {
-      if (xsane.param.depth == 16)
-      {
-        output_format = XSANE_RAW16;
-      }
-    }
-    else if ( (!strcasecmp(extension, "pnm")) || (!strcasecmp(extension, "ppm")) ||
-              (!strcasecmp(extension, "pgm")) || (!strcasecmp(extension, "pbm")) )
-    {
-      if (xsane.param.depth == 16)
+      if ((xsane.param.depth == 16) && (!preferences.reduce_16bit_to_8bit) )
       {
         output_format = XSANE_PNM16;
       }
@@ -1794,6 +1701,10 @@ int xsane_identify_output_format(char *filename, char **ext)
         output_format = XSANE_PNM;
       }
     }
+    else if ( (!strcasecmp(extension, "txt")) || (!strcasecmp(extension, "text")) )
+    {
+      output_format = XSANE_TEXT;
+    }
 #ifdef HAVE_LIBPNG
 #ifdef HAVE_LIBZ
     else if (!strcasecmp(extension, "png"))
@@ -1855,7 +1766,7 @@ void xsane_change_working_directory(void)
   sprintf(windowname, "%s %s %s", xsane.prog_name, WINDOW_CHANGE_WORKING_DIR, xsane.device_text);
   if (getcwd(filename, sizeof(filename)))
   {
-    xsane_back_gtk_get_filename(windowname, filename, sizeof(filename), filename, TRUE, FALSE, TRUE);
+    xsane_back_gtk_get_filename(windowname, filename, sizeof(filename), filename, NULL, TRUE, FALSE, TRUE, FALSE);
     if (chdir(filename))
     {
      char buf[256];
@@ -1865,6 +1776,14 @@ void xsane_change_working_directory(void)
       xsane_set_sensitivity(TRUE);
       return;
     }
+    else
+    {
+      if (preferences.working_directory)
+      {
+        free(preferences.working_directory);
+      }
+      preferences.working_directory = strdup(filename);
+    }
   }
 
   xsane_set_sensitivity(TRUE);
@@ -1875,6 +1794,18 @@ void xsane_change_working_directory(void)
 static int eula_accept_flag;
 static GtkWidget *eula_dialog = NULL;
 
+static gboolean xsane_eula_delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+{
+  eula_accept_flag = (int) data;
+
+  DBG(DBG_proc ,"xsane_eula_delete_event(%d)\n", eula_accept_flag);
+  eula_dialog = NULL;
+
+ return FALSE; /* continue with original delete even routine */
+}
+
+/* -------------------------------------- */
+
 static void xsane_eula_button_callback(GtkWidget *widget, gpointer data)
 {
   eula_accept_flag = (int) data;
@@ -1885,12 +1816,12 @@ static void xsane_eula_button_callback(GtkWidget *widget, gpointer data)
   eula_dialog = NULL;
 }
 
-/* ---------------------------------------------------------------------------------------------------------------------- */
+/* -------------------------------------- */
 
 int xsane_display_eula(int ask_for_accept)
 /* returns FALSE if accepted, TRUE if not accepted */
 {
- GtkWidget *vbox, *hbox, *button, *label;
+ GtkWidget *vbox, *hbox, *button, *label, *frame;
  GtkAccelGroup *accelerator_group;
  char buf[1024];
  char filename[PATH_MAX];
@@ -1907,18 +1838,32 @@ int xsane_display_eula(int ask_for_accept)
   gtk_widget_set_size_request(eula_dialog, 550, 580);
   gtk_window_set_position(GTK_WINDOW(eula_dialog), GTK_WIN_POS_CENTER);
   gtk_window_set_resizable(GTK_WINDOW(eula_dialog), TRUE);
-  g_signal_connect(GTK_OBJECT(eula_dialog), "delete_event", GTK_SIGNAL_FUNC(xsane_eula_button_callback), (void *) -1); /* -1 = cancel */
+  g_signal_connect(GTK_OBJECT(eula_dialog), "delete_event", GTK_SIGNAL_FUNC(xsane_eula_delete_event), (void *) -1); /* -1 = cancel */
   snprintf(buf, sizeof(buf), "%s %s", xsane.prog_name, WINDOW_EULA);
   gtk_window_set_title(GTK_WINDOW(eula_dialog), buf);
-  xsane_set_window_icon(eula_dialog, 0);
-
   accelerator_group = gtk_accel_group_new();
   gtk_window_add_accel_group(GTK_WINDOW(eula_dialog), accelerator_group);
 
+#if 0
+  xsane_set_window_icon(eula_dialog, 0);
+#endif
+
+  /* create a frame */
+  frame = gtk_frame_new(NULL);
+  gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
+  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
+  gtk_container_add(GTK_CONTAINER(eula_dialog), frame);
+  gtk_widget_show(frame);
+
   vbox = gtk_vbox_new(FALSE, 5);
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
-  gtk_container_add(GTK_CONTAINER(eula_dialog), vbox);
+  gtk_container_add(GTK_CONTAINER(frame), vbox);
   gtk_widget_show(vbox); 
+#if 1
+  /* this is normally done directly after gtk_window_set_title() */
+  /* but gtk crashes when we would do that and select a text with the mouse */
+  xsane_set_window_icon(eula_dialog, 0);
+#endif
 
   /* display XSane copyright message */
   snprintf(buf, sizeof(buf), "XSane %s %s\n"
@@ -2101,16 +2046,28 @@ static GtkWidget *license_dialog = NULL;
 
 static void xsane_close_license_dialog_callback(GtkWidget *widget, gpointer data)
 {
+  DBG(DBG_proc ,"xsane_close_license_dialog_callback\n");
+
   gtk_widget_destroy(license_dialog);
   license_dialog = NULL;
 }
 
-/* ---------------------------------------------------------------------------------------------------------------------- */
+/* ------------------------------------------------ */
 
-int xsane_display_gpl(void)
-/* returns FALSE if accepted, TRUE if not accepted */
+static gboolean xsane_license_dialog_delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
- GtkWidget *vbox, *hbox, *button, *label;
+  DBG(DBG_proc ,"xsane_license_dialog_delete_event\n");
+
+  license_dialog = NULL;
+
+ return FALSE; /* continue with original delete even routine */
+}
+
+/* ------------------------------------------------ */
+
+void xsane_display_gpl(void)
+{
+ GtkWidget *vbox, *hbox, *button, *label, *frame;
  GtkAccelGroup *accelerator_group;
  char buf[1024];
  char filename[PATH_MAX];
@@ -2118,24 +2075,39 @@ int xsane_display_gpl(void)
 
   if (license_dialog) /* make sure the dialog is only opend once */
   {
-    return 0;
+    return;
   }
 
   license_dialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_widget_set_size_request(license_dialog, 550, 580);
   gtk_window_set_position(GTK_WINDOW(license_dialog), GTK_WIN_POS_CENTER);
   gtk_window_set_resizable(GTK_WINDOW(license_dialog), TRUE);
+  g_signal_connect(GTK_OBJECT(license_dialog), "delete_event", GTK_SIGNAL_FUNC(xsane_license_dialog_delete_event), NULL);
   snprintf(buf, sizeof(buf), "%s: %s", xsane.prog_name, WINDOW_GPL);
   gtk_window_set_title(GTK_WINDOW(license_dialog), buf);
-  xsane_set_window_icon(license_dialog, 0);
-
   accelerator_group = gtk_accel_group_new();
   gtk_window_add_accel_group(GTK_WINDOW(license_dialog), accelerator_group);
 
+#if 0
+  xsane_set_window_icon(license_dialog, 0);
+#endif
+
+  /* create a frame */
+  frame = gtk_frame_new(NULL);
+  gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
+  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
+  gtk_container_add(GTK_CONTAINER(license_dialog), frame);
+  gtk_widget_show(frame);
+
   vbox = gtk_vbox_new(FALSE, 5);
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
-  gtk_container_add(GTK_CONTAINER(license_dialog), vbox);
+  gtk_container_add(GTK_CONTAINER(frame), vbox);
   gtk_widget_show(vbox); 
+#if 1
+  /* this is normally done directly after gtk_window_set_title() */
+  /* but gtk crashes when we would do that and select a text with the mouse */
+  xsane_set_window_icon(license_dialog, 0);
+#endif
 
   /* display XSane copyright message */
   snprintf(buf, sizeof(buf), "XSane %s %s\n"
@@ -2201,7 +2173,7 @@ int xsane_display_gpl(void)
     else
     {
       DBG(DBG_error0, "ERROR: license text not found. Looks like xsane is not installed correct.\n");
-     return TRUE;
+     return;
     }
 
   }
@@ -2244,7 +2216,7 @@ int xsane_display_gpl(void)
     else
     {
       DBG(DBG_error0, "ERROR: license text not found. Looks like xsane is not installed correct.\n");
-     return TRUE;
+     return;
     }
 
     /* Thaw the text widget, allowing the updates to become visible */
@@ -2273,8 +2245,6 @@ int xsane_display_gpl(void)
   gtk_widget_show(hbox);
   gtk_widget_show(vbox);
   gtk_widget_show(license_dialog);
-
-  return 0;
 }
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
@@ -2282,7 +2252,7 @@ int xsane_display_gpl(void)
 void xsane_window_get_position(GtkWidget *gtk_window, gint *x, gint *y)
 {
 #ifdef HAVE_GTK2
-  gtk_window_get_position(gtk_window, x, y);
+  gtk_window_get_position(GTK_WINDOW(gtk_window), x, y);
 #else
   if (xsane.get_deskrelative_origin)
   {
