@@ -68,12 +68,13 @@ extern void xsane_range_new_with_pixmap(GdkWindow *window, GtkBox *parent, const
 extern void xsane_set_filename(gchar *filename);
 extern void xsane_outputfilename_new(GtkWidget *vbox);
 extern void xsane_separator_new(GtkWidget *xsane_parent, int dist);
+extern void xsane_vseparator_new(GtkWidget *xsane_parent, int dist);
 extern GtkWidget *xsane_info_table_text_new(GtkWidget *table, gchar *text, int row, int colomn);
 extern GtkWidget *xsane_info_text_new(GtkWidget *parent, gchar *text);
 extern void xsane_refresh_dialog(void);
 extern void xsane_update_param(void *arg);
 extern void xsane_define_output_filename(void);
-extern int xsane_identify_output_format(char *filename, char **ext);
+extern int xsane_identify_output_format(char *filename, char *filetype, char **ext);
 extern void xsane_change_working_directory(void);
 extern int xsane_display_eula(int ask_for_accept);
 extern int xsane_display_gpl(void);
