@@ -536,6 +536,20 @@ void xsane_update_sliders()
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
+static gint xsane_batch_scan_gamma_event()
+{
+  DBG(DBG_proc, "xsane_batch_scan_gamma_event\n");
+
+  xsane_batch_scan_update_icon_list(); /* update gamma of batch scan icons */
+
+  gtk_timeout_remove(xsane.batch_scan_gamma_timer);
+  xsane.batch_scan_gamma_timer = 0;
+
+ return FALSE;
+}
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
 static gint xsane_slider_hold_event()
 {
   DBG(DBG_proc, "xsane_slider_hold_event\n");
@@ -1114,9 +1128,9 @@ void xsane_create_preview_gamma_curve(u_char *gammadata, int negative, double ga
  double val;
  double m;
  double b;
- int maxin = numbers-1;
  double medium_m;
  double medium_mid;
+ int maxin = numbers-1;
 
   medium_m   = 100.0/(medium_highlight - medium_shadow);
   medium_mid = (medium_shadow + medium_highlight)/200.0 * maxin;
@@ -1152,7 +1166,7 @@ void xsane_create_preview_gamma_curve(u_char *gammadata, int negative, double ga
       val = val * m + b;
       xsane_bound_double(&val, 0.0, maxin);
 
-      gammadata[i] = (u_char) (255 * pow( val/maxin, (1.0/gamma) ));
+      gammadata[i] = (u_char) (255.99999 * pow( ceil(val)/maxin, (1.0/gamma) ));
     }
   }
   else /* positive */
@@ -1172,7 +1186,7 @@ void xsane_create_preview_gamma_curve(u_char *gammadata, int negative, double ga
       val = val * m + b;
       xsane_bound_double(&val, 0.0, maxin);
 
-      gammadata[i] = (u_char) (255 * pow( val/maxin, (1.0/gamma) ));
+      gammadata[i] = (u_char) (255.99999 * pow( val/maxin, (1.0/gamma) ));
     }
   }
 }
@@ -1230,7 +1244,7 @@ void xsane_create_gamma_curve(SANE_Int *gammadata,
       /* user correction */
       val = val * m + b;
       xsane_bound_double(&val, 0.0, maxin);
-      gammadata[i] = maxout * pow( val/maxin, (1.0/gamma) );
+      gammadata[i] = (int) (maxout * pow( val/maxin, (1.0/gamma) ));
     }
   }
   else /* positive */
@@ -1249,7 +1263,7 @@ void xsane_create_gamma_curve(SANE_Int *gammadata,
       /* user correction */
       val = val * m + b;
       xsane_bound_double(&val, 0.0, maxin);
-      gammadata[i] = maxout * pow( val/maxin, (1.0/gamma) );
+      gammadata[i] = (int) (maxout * pow( val/maxin, (1.0/gamma) ));
     }
   }
 }
@@ -1737,6 +1751,12 @@ void xsane_enhancement_by_gamma(void)
 
   xsane_enhancement_update();
   xsane_update_gamma_curve(FALSE);
+
+  if (xsane.batch_scan_gamma_timer)
+  {
+    gtk_timeout_remove(xsane.batch_scan_gamma_timer);
+  }
+  xsane.batch_scan_gamma_timer = gtk_timeout_add(XSANE_CONTINUOUS_HOLD_TIME * 4, xsane_batch_scan_gamma_event, 0);
 }
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
@@ -2362,6 +2382,7 @@ void xsane_set_auto_enhancement()
 }
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
+
 void xsane_set_medium(Preferences_medium_t *medium)
 {
  const SANE_Option_Descriptor *opt;
