@@ -59,153 +59,269 @@ const char *error_xpm[] =
 "40 40 6 1",
 " 	c None",
 ".	c #FE0809",
-"+	c #FDFDFC",
-"@	c #E5D5D6",
-"#	c #E1A09F",
-"$	c #FE8081",
-"               #@@++++@@#               ",
-"            #@++++++++++++@#            ",
-"          #+++@#$......$$@+++#          ",
-"        #+++#..............#+++#        ",
-"       #++#..................@++#       ",
-"      @++$....................$++@      ",
-"     @+@........................++@     ",
-"    #+@$.........................@+#    ",
-"   #++...........................$++#   ",
-"   ++$.....@+...............++....$++   ",
-"  #+#.....+++@.............+++@....#+#  ",
-"  ++.....$+++++...........@++++$....++  ",
-" #+#......#++++@.........+++++#.....#+# ",
-" @+........#+++++.......@++++$.......+@ ",
-" +@.........$++++@.....+++++#........++ ",
-"#+$..........#+++++...@++++$.........$+#",
-"@+$...........$+++++$@++++#..........$+@",
-"@+.............#+++++++++$............+@",
-"++..............$+++++++#.............++",
-"++...............#+++++#..............++",
-"++...............++++++@..............++",
-"++..............@++++++++.............++",
-"@+.............+++++@++++@............+@",
-"@+$...........@++++#.#+++++..........$+@",
-"#+$..........+++++$...$++++@.........$+#",
-" +@.........@++++#.....#+++++........@+ ",
-" @+........+++++$.......$++++@.......+@ ",
-" #+#......@++++#.........#+++++.....#+# ",
-"  ++.....#++++#...........$++++$....++  ",
-"  @+#.....$++$.............#++$....#+#  ",
-"   ++$.....#$...............$#....$++   ",
-"   #++............................++#   ",
-"    #++..........................++#    ",
-"     @++........................++@     ",
-"      @++$....................$++@      ",
-"       #++#..................#++#       ",
-"        #+++#..............#+++#        ",
-"          #+++@$$......$$++++#          ",
-"            #+++++++++++++@#            ",
-"               #@@++++@@#               "
+"+	c #FFFFFF",
+"@	c #FF8080",
+"-      c #7A7A7A",
+"#      c #6C6B6B",
+"                ----------              ",
+"             -- @@@+@@@--###            ",
+"           --@@+++++++++@@--##          ",
+"         --@+++@@@...@@@+++@--##        ",
+"        -@+++@...........@+++@-##       ",
+"       -@++@...............@++@-##      ",
+"      -@++@.................@++@-##     ",
+"     -@+@.....................++@-##    ",
+"    -@+@@......................@+@-##   ",
+"   -@++........................@++@-#   ",
+"  --++@.....@+............++....@++-##  ",
+"  -@+@.....+++@..........+++@....@+@-#  ",
+" --++.....@+++++........@++++@....++-## ",
+" -@+@......@++++@......+++++@.....@+@-# ",
+" -@+........@+++++....@++++@.......+@-# ",
+" -+@.........@++++@..+++++@........++-##",
+"-@+@..........@+++++@++++@.........@+@-#",
+"-@+@...........@++++++++@..........@+@-#",
+"-@+.............@++++++@............+@-#",
+"-++..............@++++@.............++-#",
+"-@+.............+++++++@............+@-#",
+"-@+@...........@+++++++++..........@+@-#",
+"-@+@..........+++++@@++++@.........@+@-#",
+" -+@.........@++++@..@+++++........@+-##",
+" -@+........+++++@....@++++@.......+@-# ",
+" -@+@......@++++@......@+++++.....@+@-# ",
+"  -++.....@++++@........@++++@....++-## ",
+"  -@+@.....@++@..........@++@....@+@-#  ",
+"   -++@.....@@............@@....@++-##  ",
+"   -@++.........................++@-#   ",
+"    -@++.......................++@-##   ",
+"     -@++.....................++@-##    ",
+"     --@++@.................@++@-##     ",
+"      --@++@...............@++@-##      ",
+"       --@+++@...........@+++@-##       ",
+"        ---@+++@@@...@@++++@--##        ",
+"          ---@++++++++++@@--###         ",
+"            ----@@@+@@@---###           ",
+"             ----------####             ",
+"               --#######                "
 };
 
 /* --------------------------------------------------- */
 
-const char *warning_xpm[] =
-{
-"40 40 4 1",
-" 	c None",
-".	c #FFFFFF",
-"+	c #FFFA00",
-"@	c #000000",
-"                                        ",
-"                   ..                   ",
-"                   ..                   ",
-"                  .++.                  ",
-"                  .++.                  ",
-"                 .++++.                 ",
-"                 .++++.                 ",
-"                .++++++.                ",
-"                .++++++.                ",
-"               .++++++++.               ",
-"               .++++++++.               ",
-"              .++++++++++.              ",
-"              .++++@@++++.              ",
-"             .++++@@@@++++.             ",
-"             .+++@@@@@@+++.             ",
-"            .++++@@@@@@++++.            ",
-"            .+++@@@@@@@@+++.            ",
-"           .++++@@@@@@@@++++.           ",
-"           .++++@@@@@@@@++++.           ",
-"          .++++++@@@@@@++++++.          ",
-"          .++++++@@@@@@++++++.          ",
-"         .+++++++@@@@@@+++++++.         ",
-"         .+++++++@@@@@@+++++++.         ",
-"        .+++++++++@@@@+++++++++.        ",
-"        .+++++++++@@@@+++++++++.        ",
-"       .++++++++++@@@@++++++++++.       ",
-"       .+++++++++++@@+++++++++++.       ",
-"      .++++++++++++++++++++++++++.      ",
-"      .++++++++++++++++++++++++++.      ",
-"     .++++++++++++++++++++++++++++.     ",
-"     .+++++++++++++@@+++++++++++++.     ",
-"    .+++++++++++++@@@@+++++++++++++.    ",
-"    .+++++++++++++@@@@+++++++++++++.    ",
-"   .+++++++++++++++@@+++++++++++++++.   ",
-"   .++++++++++++++++++++++++++++++++.   ",
-"  .++++++++++++++++++++++++++++++++++.  ",
-"  .++++++++++++++++++++++++++++++++++.  ",
-" ..++++++++++++++++++++++++++++++++++.. ",
-" ...................................... ",
-"........................................"
-};
+const char *warning_xpm[] = {
+"40 40 6 1",
+"       c None",
+".      c #FFFFFF",
+"+      c #FFFA00",
+"@      c #000000",
+"-      c #7A7A7A",
+"#      c #6C6B6B",
+"                 ----##                 ",
+"                 -..-###                ",
+"                -....-##                ",
+"                -....-###               ",
+"               -..++..-##               ",
+"               -..++..-###              ",
+"              -..++++..-##              ",
+"              -..++++..-###             ",
+"             -..++++++..-##             ",
+"             -..++++++..-###            ",
+"            -..++++++++..-##            ",
+"            -..+++@@+++..-###           ",
+"           -..+++@@@@+++..-##           ",
+"           -..++@@@@@@++..-###          ",
+"          -..+++@@@@@@+++..-##          ",
+"          -..++@@@@@@@@++..-###         ",
+"         -..+++@@@@@@@@+++..-##         ",
+"         -..+++@@@@@@@@+++..-###        ",
+"        -..+++++@@@@@@+++++..-##        ",
+"        -..+++++@@@@@@+++++..-###       ",
+"       -..++++++@@@@@@++++++..-##       ",
+"       -..++++++@@@@@@++++++..-###      ",
+"      -..++++++++@@@@++++++++..-##      ",
+"      -..++++++++@@@@++++++++..-###     ",
+"     -..+++++++++@@@@+++++++++..-##     ",
+"     -..++++++++++@@++++++++++..-###    ",
+"    -..++++++++++++++++++++++++..-##    ",
+"    -..++++++++++++++++++++++++..-###   ",
+"   -..++++++++++++++++++++++++++..-##   ",
+"   -..++++++++++++@@++++++++++++..-###  ",
+"  -..++++++++++++@@@@++++++++++++..-##  ",
+"  -..++++++++++++@@@@++++++++++++..-##  ",
+" -..++++++++++++++@@++++++++++++++..-## ",
+" -..++++++++++++++++++++++++++++++..-## ",
+"-..++++++++++++++++++++++++++++++++..-##",
+"-....................................-##",
+"-....................................-##",
+"--------------------------------------##",
+" #######################################",
+"  ##################################### "};
 
 /* --------------------------------------------------- */
 
-const char *info_xpm[] =
-{
-"40 40 4 1",
-" 	c None",
-".	c #FFFFFF",
+const char *info_xpm[] = {
+"40 40 6 1",
+"       c None",
+".      c #FFFFFF",
 "+	c #00A000",
-"@	c #000000",
+"@      c #000000",
+"-      c #7A7A7A",
+"#      c #6C6B6B",
+"                 ----##                 ",
+"                 -..-###                ",
+"                -....-##                ",
+"                -....-###               ",
+"               -..++..-##               ",
+"               -..++..-###              ",
+"              -..++++..-##              ",
+"              -..++++..-###             ",
+"             -..++++++..-##             ",
+"             -..++++++..-###            ",
+"            -..++++++++..-##            ",
+"            -..+++@@+++..-###           ",
+"           -..+++@@@@+++..-##           ",
+"           -..++@@@@@@++..-###          ",
+"          -..+++@@@@@@+++..-##          ",
+"          -..++@@@@@@@@++..-###         ",
+"         -..+++@@@@@@@@+++..-##         ",
+"         -..+++@@@@@@@@+++..-###        ",
+"        -..+++++@@@@@@+++++..-##        ",
+"        -..+++++@@@@@@+++++..-###       ",
+"       -..++++++@@@@@@++++++..-##       ",
+"       -..++++++@@@@@@++++++..-###      ",
+"      -..++++++++@@@@++++++++..-##      ",
+"      -..++++++++@@@@++++++++..-###     ",
+"     -..+++++++++@@@@+++++++++..-##     ",
+"     -..++++++++++@@++++++++++..-###    ",
+"    -..++++++++++++++++++++++++..-##    ",
+"    -..++++++++++++++++++++++++..-###   ",
+"   -..++++++++++++++++++++++++++..-##   ",
+"   -..++++++++++++@@++++++++++++..-###  ",
+"  -..++++++++++++@@@@++++++++++++..-##  ",
+"  -..++++++++++++@@@@++++++++++++..-##  ",
+" -..++++++++++++++@@++++++++++++++..-## ",
+" -..++++++++++++++++++++++++++++++..-## ",
+"-..++++++++++++++++++++++++++++++++..-##",
+"-....................................-##",
+"-....................................-##",
+"--------------------------------------##",
+" #######################################",
+"  ##################################### "};
+
+/* --------------------------------------------------- */
+
+const char *device_xpm[] = {
+"40 40 12 1",
+"       c None",
+".      c #FFFFFF",
+"*	c #FAFAFA",
+"+	c #F8F8F8",
+"0	c #F2F2F2",
+"-      c #808080",
+"_      c #505050",
+"@      c #000000",
+"A      c #A0E0F8",
+"B      c #A8E8FA",
+"C      c #B0F0FF",
+"D      c #C0F8FF",
 "                                        ",
-"                   ..                   ",
-"                   ..                   ",
-"                  .++.                  ",
-"                  .++.                  ",
-"                 .++++.                 ",
-"                 .++++.                 ",
-"                .++++++.                ",
-"                .++++++.                ",
-"               .++++++++.               ",
-"               .++++++++.               ",
-"              .++++++++++.              ",
-"              .++++@@++++.              ",
-"             .++++@@@@++++.             ",
-"             .+++@@@@@@+++.             ",
-"            .++++@@@@@@++++.            ",
-"            .+++@@@@@@@@+++.            ",
-"           .++++@@@@@@@@++++.           ",
-"           .++++@@@@@@@@++++.           ",
-"          .++++++@@@@@@++++++.          ",
-"          .++++++@@@@@@++++++.          ",
-"         .+++++++@@@@@@+++++++.         ",
-"         .+++++++@@@@@@+++++++.         ",
-"        .+++++++++@@@@+++++++++.        ",
-"        .+++++++++@@@@+++++++++.        ",
-"       .++++++++++@@@@++++++++++.       ",
-"       .+++++++++++@@+++++++++++.       ",
-"      .++++++++++++++++++++++++++.      ",
-"      .++++++++++++++++++++++++++.      ",
-"     .++++++++++++++++++++++++++++.     ",
-"     .+++++++++++++@@+++++++++++++.     ",
-"    .+++++++++++++@@@@+++++++++++++.    ",
-"    .+++++++++++++@@@@+++++++++++++.    ",
-"   .+++++++++++++++@@+++++++++++++++.   ",
-"   .++++++++++++++++++++++++++++++++.   ",
-"  .++++++++++++++++++++++++++++++++++.  ",
-"  .++++++++++++++++++++++++++++++++++.  ",
-" ..++++++++++++++++++++++++++++++++++.. ",
-" ...................................... ",
-"........................................"
-};
+"                                        ",
+"                                        ",
+"                     --                 ",
+"                  ---..---              ",
+"               ---.....*---             ",
+"             --.......***---     @@@    ",
+"             _-......****+---  @@   @   ",
+"              _-....****+++---@      @  ",
+"               _-..****+++++-@-      @  ",
+"                _-****+++++++---  @@@   ",
+"                -_-**+++++++++---       ",
+"               -.._-+++++++++++---      ",
+"              -...A_-++++++++000---     ",
+"             -...AAA_-++++++00000---    ",
+"            -...AAAAA_-++++0000000---   ",
+"           -...AAAAAAA_-++000000000---  ",
+"          -...AAAAAAAAA_-00000000000--- ",
+"         -...AAAAAAAAAAB_-000000000-_---",
+"        -...AAAAAAAAAABBB_-000000--_----",
+"       -...AAAAAAAAAABBBBB_-000--_-+_---",
+"      -...AAAAAAAAAABBBBCCC_---_--+_--- ",
+"     -...AAAAAAAAAABBBBCCCCC__---+_---  ",
+"    -+...AAAAAAAAABBBBCCCCDDD---+_---   ",
+"     -+..*AAAAAAABBBBCCCCDDD..-+_---    ",
+"      -+***AAAAABBBBCCCCDDD000+_---     ",
+"       -+**AAAABBBBCCCCDDD000+_---      ",
+"        -+**AABBBBCCCCDDD000+_---       ",
+"         -+++BBBBCCCCDDD000+_---        ",
+"          -++0BBCCCCDDD000+_---         ",
+"           -+00CCCCDDD000+_---          ",
+"            -+000CDDD000+_---           ",
+"             -+000DD000+_---            ",
+"              -+0++000+_---             ",
+"               -++++++_---              ",
+"                -++++_---               ",
+"                 -++_---                ",
+"                  -----                 ",
+"                   ---                  ",
+"                                        "};
+
+/* --------------------------------------------------- */
+
+const char *no_device_xpm[] = {
+"40 40 13 1",
+"       c None",
+".      c #FFFFFF",
+"*	c #FAFAFA",
+"+	c #F8F8F8",
+"0	c #F2F2F2",
+"-      c #808080",
+"_      c #505050",
+"@      c #000000",
+"A      c #A0E0F8",
+"B      c #A8E8FA",
+"C      c #B0F0FF",
+"D      c #C0F8FF",
+"#      c #C03030",
+"                                        ",
+"                                        ",
+"                                        ",
+"                     --                 ",
+"                  ---..---              ",
+"      ##       ---.....*---     ##      ",
+"      ####   --.......***---  ####      ",
+"       ####  _-......****+---####       ",
+"        ####  _-....****+++-####        ",
+"         ####  _-..****++++####         ",
+"          ####  _-****++++####--        ",
+"           #### -_-**++++####+---       ",
+"            ####.._-++++####+++---      ",
+"             ####.A_-++####++000---     ",
+"             -####AA_-####++00000---    ",
+"            -..####AA####++0000000---   ",
+"           -...A########++000000000---  ",
+"          -...AAA######_-00000000000--- ",
+"         -...AAAAA####AA_-000000000-_---",
+"        -...AAAAA######BB_-000000--_----",
+"       -...AAAAA########BB_-000--_-+_---",
+"      -...AAAAA####A####CCC_---_--+_--- ",
+"     -...AAAAA####ABB####CCC__---+_---  ",
+"    -+...AAAA####ABBBB####DDD---+_---   ",
+"     -+..*AA####ABBBBCC####D..-+_---    ",
+"      -+***####ABBBBCCCC####00+_---     ",
+"       -+*####ABBBBCCCCDD####+_---      ",
+"        -####ABBBBCCCCDDD0####---       ",
+"        ####+BBBBCCCCDDD000####-        ",
+"       ####+00BBCCCCDDD000+_####        ",
+"      #### -+00CCCCDDD000+_--####       ",
+"      ##    -+000CDDD000+_---  ##       ",
+"             -+000DD000+_---            ",
+"              -+0++000+_---             ",
+"               -++++++_---              ",
+"                -++++_---               ",
+"                 -++_---                ",
+"                  -----                 ",
+"                   ---                  ",
+"                                        "};
+
 /* --------------------------------------------------- */
 
 const char *file_xpm[] =
@@ -242,6 +358,76 @@ const char *file_xpm[] =
 
 /* --------------------------------------------------- */
 
+const char *load_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        5            1",
+/* colors */
+"* c #000000",
+". c #ffff70",
+"+ c #505000",
+"- c #e0e0e0",
+"  none",
+/* pixels */
+"                    ",
+"     **             ",
+"    *..****         ",
+"    *......****     ",
+"    *..........**** ",
+" ****..............*",
+"*---++++...........*",
+"*-------++++.......*",
+"*-----------++++...*",
+"*---------------+..*",
+" *--------------+..*",
+" *--------------+..*",
+" *---------------+.*",
+"  *--------------+.*",
+"  *--------------+.*",
+"  *---------------+*",
+"   ****-----------+*",
+"       ****-------+*",
+"           *********",
+"                    "
+};
+
+/* --------------------------------------------------- */
+
+const char *save_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        5            1",
+/* colors */
+"* c #000000",
+". c #70b070",
+"+ c #609060",
+"- c #e0e0e0",
+"  none",
+/* pixels */
+"                    ",
+"   ................ ",
+"   .*............*. ",
+"   .*............*. ",
+"   ................ ",
+"   ................ ",
+"   ................ ",
+"   ................ ",
+"   ................ ",
+"   ................ ",
+"   ................ ",
+"   ................ ",
+"   ....--------.... ",
+"   ....-+++----.... ",
+"   ....-+++----.... ",
+"   ....-+++----.... ",
+"   ....-+++----.... ",
+"    ............... ",
+"                    ",
+"                    "
+};
+
+/* --------------------------------------------------- */
+
 const char *ocr_xpm[] =
 {
 /* width height num_colors chars_per_pixel */
@@ -1521,6 +1707,137 @@ const char *default_enhancement_xpm[] =
 
 /* --------------------------------------------------- */
 
+const char *add_batch_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        3            1",
+/* colors */
+". c #000000",
+"  none",
+"g c #70a070",
+/* pixels */
+"                    ",
+" ... ... ... ... .. ",
+" .                . ",
+" .                . ",
+"                    ",
+" .                . ",
+" .       gg       . ",
+" .       gg       . ",
+"         gg         ",
+" .       gg       . ",
+" .   gggggggggg   . ",
+" .   gggggggggg   . ",
+"         gg         ",
+" .       gg       . ",
+" .       gg       . ",
+" .       gg       . ",
+"                    ",
+" .                . ",
+" .                . ",
+" ... ... ... ... .. ",
+};
+
+/* --------------------------------------------------- */
+
+const char *del_batch_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        3            1",
+/* colors */
+". c #000000",
+"  none",
+"r c #d07070",
+/* pixels */
+"                    ",
+"rr.. ... ... ... .rr",
+" rr              rr ",
+" .rr            rr. ",
+"   rr          rr   ",
+" .  rr        rr  . ",
+" .   rr      rr   . ",
+" .    rr    rr    . ",
+"       rr  rr       ",
+" .      rrrr      . ",
+" .       rr       . ",
+" .      rrrr      . ",
+"       rr  rr       ",
+" .    rr    rr    . ",
+" .   rr      rr   . ",
+" .  rr        rr  . ",
+"   rr          rr   ",
+" .rr            rr. ",
+" rr              rr ",
+"rr.. ... ... ... .rr",
+};
+
+/* --------------------------------------------------- */
+
+const char *empty_batch_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        3            1",
+/* colors */
+". c #000000",
+"  none",
+"* c #ffffff",
+/* pixels */
+"                    ",
+"  ................  ",
+"  .**************.  ",
+"  .**************.  ",
+"  .**************.  ",
+"  .**************.  ",
+"  .**************.  ",
+"  .**************.  ",
+"  .**************.  ",
+"  .**************.  ",
+"  .**************.  ",
+"  .**************.  ",
+"  .**************.  ",
+"  .**************.  ",
+"  .**************.  ",
+"  .**************.  ",
+"  .**************.  ",
+"  .**************.  ",
+"  .**************.  ",
+"  ................  ",
+};
+
+/* --------------------------------------------------- */
+
+const char *ascii_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        2            1",
+/* colors */
+". c #000000",
+"  none",
+/* pixels */
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+" .....  ....   ...  ",
+" .   .  .   . .   . ",
+" .   .  .   . .     ",
+" .....  ..... .     ",
+" .   .  .   . .     ",
+" .   .  .   . .   . ",
+" .   .  ....   ...  ",
+"                    ",
+" .................. ",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
 const char *pipette_white_xpm[] =
 {
 /* width height num_colors chars_per_pixel */
@@ -1950,6 +2267,175 @@ const char *delete_images_xpm[] =
 
 /* --------------------------------------------------- */
 
+const char *size_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+"  none",
+". c #000000",
+"+ c #c0c0c0",
+"a c #ffffff",
+/* pixels */
+"                    ",
+" .................. ",
+" .                . ",
+" .           .    . ",
+" .                . ",
+" .           .    . ",
+" .                . ",
+" .           .    . ",
+" .                . ",
+" . . . . . . .    . ",
+" .                . ",
+" .                . ",
+" .                . ",
+" .                . ",
+" .                . ",
+" .                . ",
+" .                . ",
+" .                . ",
+" .................. ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
+const char *size_x_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+"  none",
+". c #000000",
+"+ c #8080f0",
+"a c #ffffff",
+/* pixels */
+"                    ",
+" .................. ",
+" .                . ",
+" .                . ",
+" .                . ",
+" .                . ",
+" .                . ",
+" .  +          +  . ",
+" . ++          ++ . ",
+" .++++++++++++++++. ",
+" .++++++++++++++++. ",
+" . ++          ++ . ",
+" .  +          +  . ",
+" .                . ",
+" .                . ",
+" .                . ",
+" .                . ",
+" .                . ",
+" .................. ",
+"                    ",
+};
+/* --------------------------------------------------- */
+
+const char *size_y_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+"  none",
+". c #000000",
+"+ c #8080f0",
+"a c #ffffff",
+/* pixels */
+"                    ",
+" .................. ",
+" .       ++       . ",
+" .      ++++      . ",
+" .     ++++++     . ",
+" .       ++       . ",
+" .       ++       . ",
+" .       ++       . ",
+" .       ++       . ",
+" .       ++       . ",
+" .       ++       . ",
+" .       ++       . ",
+" .       ++       . ",
+" .       ++       . ",
+" .       ++       . ",
+" .     ++++++     . ",
+" .      ++++      . ",
+" .       ++       . ",
+" .................. ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
+const char *rotation_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+"  none",
+". c #000000",
+"+ c #c0c0c0",
+"a c #ffffff",
+/* pixels */
+"                    ",
+" ..                 ",
+" ..                 ",
+" ..                 ",
+" ..                 ",
+" ..                 ",
+" .......            ",
+" ..     .           ",
+" ..      .          ",
+" ..       .         ",
+" ..       .         ",
+" ..        .        ",
+" ..        .        ",
+" ..        .        ",
+" ..        .        ",
+" ..      .....      ",
+" ..       ...       ",
+" ... . . . . . . .  ",
+" .. . . . . . . . . ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
+const char *aspect_ratio_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+"  none",
+". c #000000",
+"+ c #c0c0c0",
+"a c #ffffff",
+/* pixels */
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+" .................  ",
+" .               .  ",
+" . .             .  ",
+" .   .           .  ",
+" .     .         .  ",
+" .       .       .  ",
+" .         .     .  ",
+" .           .   .  ",
+" .             . .  ",
+" .................  ",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
 const char *printer_xpm[] =
 {
 /* width height num_colors chars_per_pixel */
@@ -2613,6 +3099,346 @@ const char *incomplete_xpm[] =
 
 /* --------------------------------------------------- */
 
+const char *paper_orientation_portrait_center_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+". c #00A000",
+"+ c #000000",
+"- c #ffffff",
+"  none",
+/* pixels */
+"    ++++++++++++    ",
+"    ++++++++++++    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +---....---+    ",
+"    +---....---+    ",
+"    +---....---+    ",
+"    +---....---+    ",
+"    +---....---+    ",
+"    +---....---+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    ++++++++++++    ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
+const char *paper_orientation_portrait_top_left_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+". c #00A000",
+"+ c #000000",
+"- c #ffffff",
+"  none",
+/* pixels */
+"    ++++++++++++    ",
+"    ++++++++++++    ",
+"    +....------+    ",
+"    +....------+    ",
+"    +....------+    ",
+"    +....------+    ",
+"    +....------+    ",
+"    +....------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    ++++++++++++    ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
+const char *paper_orientation_portrait_top_right_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+". c #00A000",
+"+ c #000000",
+"- c #ffffff",
+"  none",
+/* pixels */
+"    ++++++++++++    ",
+"    ++++++++++++    ",
+"    +------....+    ",
+"    +------....+    ",
+"    +------....+    ",
+"    +------....+    ",
+"    +------....+    ",
+"    +------....+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    ++++++++++++    ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
+const char *paper_orientation_portrait_bottom_left_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+". c #00A000",
+"+ c #000000",
+"- c #ffffff",
+"  none",
+/* pixels */
+"    ++++++++++++    ",
+"    ++++++++++++    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +....------+    ",
+"    +....------+    ",
+"    +....------+    ",
+"    +....------+    ",
+"    +....------+    ",
+"    +....------+    ",
+"    ++++++++++++    ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
+const char *paper_orientation_portrait_bottom_right_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+". c #00A000",
+"+ c #000000",
+"- c #ffffff",
+"  none",
+/* pixels */
+"    ++++++++++++    ",
+"    ++++++++++++    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +----------+    ",
+"    +------....+    ",
+"    +------....+    ",
+"    +------....+    ",
+"    +------....+    ",
+"    +------....+    ",
+"    +------....+    ",
+"    ++++++++++++    ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
+const char *paper_orientation_landscape_center_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+". c #00A000",
+"+ c #000000",
+"- c #ffffff",
+"  none",
+/* pixels */
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+" +++++++++++++++++++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +----........----++",
+" +----........----++",
+" +----........----++",
+" +----........----++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +++++++++++++++++++",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
+const char *paper_orientation_landscape_top_left_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+". c #00A000",
+"+ c #000000",
+"- c #ffffff",
+"  none",
+/* pixels */
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+" +++++++++++++++++++",
+" +........--------++",
+" +........--------++",
+" +........--------++",
+" +........--------++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +++++++++++++++++++",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
+const char *paper_orientation_landscape_top_right_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+". c #00A000",
+"+ c #000000",
+"- c #ffffff",
+"  none",
+/* pixels */
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+" +++++++++++++++++++",
+" +--------........++",
+" +--------........++",
+" +--------........++",
+" +--------........++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +++++++++++++++++++",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
+const char *paper_orientation_landscape_bottom_left_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+". c #00A000",
+"+ c #000000",
+"- c #ffffff",
+"  none",
+/* pixels */
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+" +++++++++++++++++++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +........--------++",
+" +........--------++",
+" +........--------++",
+" +........--------++",
+" +++++++++++++++++++",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
+const char *paper_orientation_landscape_bottom_right_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+". c #00A000",
+"+ c #000000",
+"- c #ffffff",
+"  none",
+/* pixels */
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+" +++++++++++++++++++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +----------------++",
+" +--------........++",
+" +--------........++",
+" +--------........++",
+" +--------........++",
+" +++++++++++++++++++",
+"                    ",
+"                    ",
+"                    ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
 const unsigned char cursor_pipette_white[] =
 {
    0x00, 0x70, 0x00, 0xf8, 0x80, 0xff, 0x00, 0xfe, 0x00, 0x7d, 0x80, 0x38,
