@@ -42,7 +42,11 @@ extern const char *xsane_window_icon_xpm[];
 extern const char *error_xpm[];
 extern const char *warning_xpm[];
 extern const char *info_xpm[];
+extern const char *device_xpm[];
+extern const char *no_device_xpm[];
 extern const char *file_xpm[];
+extern const char *load_xpm[];
+extern const char *save_xpm[];
 extern const char *ocr_xpm[];
 extern const char *scale_xpm[];
 extern const char *despeckle_xpm[];
@@ -80,6 +84,11 @@ extern const char *enhance_xpm[];
 extern const char *store_enhancement_xpm[];
 extern const char *restore_enhancement_xpm[];
 extern const char *default_enhancement_xpm[];
+extern const char *add_batch_xpm[];
+extern const char *del_batch_xpm[];
+extern const char *empty_batch_xpm[];
+extern const char *ascii_xpm[];
+
 extern const char *pipette_white_xpm[];
 extern const char *pipette_gray_xpm[];
 extern const char *pipette_black_xpm[];
@@ -92,6 +101,11 @@ extern const char *full_preview_area_xpm[];
 extern const char *auto_select_preview_area_xpm[];
 extern const char *auto_raise_preview_area_xpm[];
 extern const char *delete_images_xpm[];
+extern const char *size_xpm[];
+extern const char *size_x_xpm[];
+extern const char *size_y_xpm[];
+extern const char *rotation_xpm[];
+extern const char *aspect_ratio_xpm[];
 extern const char *printer_xpm[];
 extern const char *zoom_xpm[];
 extern const char *zoom_x_xpm[];
@@ -112,6 +126,16 @@ extern const char *scanning_xpm[];
 extern const char *invalid_xpm[];
 extern const char *valid_xpm[];
 extern const char *incomplete_xpm[];
+extern const char *paper_orientation_portrait_center_xpm[];
+extern const char *paper_orientation_portrait_top_left_xpm[];
+extern const char *paper_orientation_portrait_top_right_xpm[];
+extern const char *paper_orientation_portrait_bottom_left_xpm[];
+extern const char *paper_orientation_portrait_bottom_right_xpm[];
+extern const char *paper_orientation_landscape_center_xpm[];
+extern const char *paper_orientation_landscape_top_left_xpm[];
+extern const char *paper_orientation_landscape_top_right_xpm[];
+extern const char *paper_orientation_landscape_bottom_left_xpm[];
+extern const char *paper_orientation_landscape_bottom_right_xpm[];
 extern const char cursor_pipette_white[];
 extern const char cursor_pipette_gray[];
 extern const char cursor_pipette_black[];
