@@ -29,6 +29,7 @@
 
 #include <sys/types.h>
 #include <sane/sane.h>
+#include "xsane-batch-scan.h"
 
 #define SELECTION_RANGE_IN  4
 #define SELECTION_RANGE_OUT 8
@@ -48,11 +49,13 @@ enum
 
 /* ------------------------------------------------------------------------------------------------------ */
 
+#if 0
 typedef struct Batch_selection
 {
   float coordinate[4]; /* batch selection coordinate (device coord) */
   struct Batch_selection *next;
 } Batch_selection;
+#endif
 
 typedef struct
 {
@@ -87,6 +90,8 @@ typedef struct
 
   float maximum_output_width;	/* maximum output width (photocopy) */
   float maximum_output_height;	/* maximum output height (photocopy) */
+  int paper_orientation;	/* orientation of the paper (photocopy) */
+  int block_update_maximum_output_size_clipping; /* do not clip maximum output size */
 
   int index_xmin, index_xmax, index_ymin, index_ymax; /* index numbers in dependance of p->rotation */
 
@@ -139,6 +144,7 @@ typedef struct
   int selection_ypos;
   int selection_xedge;
   int selection_yedge;
+  float ratio;
 
   Tselection selection;				/* selected area to scan */
   Tselection previous_selection;		/* previous ... */
@@ -147,7 +153,9 @@ typedef struct
 
   int show_selection;
 
+#if 0
   Batch_selection *batch_selection;
+#endif
 
   GtkWidget *top;		/* top-level widget */
   GtkWidget *unit_label;
@@ -159,6 +167,8 @@ typedef struct
   GtkWidget *cancel;		/* the cancel button */
 
   GtkWidget *button_box;	/* hbox for the following buttons */
+  GtkWidget *menu_box;		/* hbox for the following menues */
+  GtkWidget *add_batch;		/* add batch button */
   GtkWidget *pipette_white;	/* pipette white button */
   GtkWidget *pipette_gray;	/* pipette gray button */
   GtkWidget *pipette_black;	/* pipette black button */
@@ -172,10 +182,12 @@ typedef struct
   GtkWidget *autoselect;	/* autoselect scanarea */
   GtkWidget *preset_area_option_menu;	/* menu for selection of preview area */
   GtkWidget *rotation_option_menu;	/* menu for selection of rotation */
+  GtkWidget *ratio_option_menu;	/* menu for selection of ratio */
   GtkWidget *scanning_pixmap;	/* pixmap that shows preview is in scanning progress */
   GtkWidget *valid_pixmap;	/* pixmap that shows preview is valid */
   GtkWidget *invalid_pixmap;	/* pixmap that shows preview is invalid */
   GtkWidget *incomplete_pixmap;	/* pixmap that shows preview is incomplete */
+  GtkWidget *rgb_label;		/* label to show RGB values */
 }
 Preview;
 
@@ -199,9 +211,10 @@ extern void preview_calculate_raw_histogram(Preview *p, SANE_Int *count_raw, SAN
 extern void preview_calculate_enh_histogram(Preview *p, SANE_Int *count, SANE_Int *count_red, SANE_Int *count_green, SANE_Int *count_blue);
 
 extern void preview_area_resize(Preview *p);					/* redraw preview rulers */
-extern void preview_set_maximum_output_size(Preview *p, float width, float height);   /* set maximum outut size */
+extern void preview_set_maximum_output_size(Preview *p, float width, float height, int paper_orientation); /* set maximum outut size */
 extern void preview_select_full_preview_area(Preview *p);
 extern void preview_display_valid(Preview *p);
+extern void preview_create_batch_icon(Preview *p, Batch_Scan_Parameters *parameters);
 
 /* ------------------------------------------------------------------------------------------------------ */
 
