@@ -24,13 +24,8 @@
 
 #include "xsane.h"
 
-#include <errno.h>
-#include <stdlib.h>
-#include <string.h>
-#include <sys/types.h>
 #include <sane/sane.h>
 #include <ctype.h>
-#include <unistd.h>
 
 #ifdef HAVE_LIBC_H
 # include <libc.h>	/* NeXTStep/OpenStep */
