@@ -50,7 +50,7 @@ static int xsane_test_multi_scan(void);
 void xsane_scan_done(SANE_Status status);
 void xsane_cancel(void);
 static void xsane_start_scan(void);
-void xsane_scan_dialog(GtkWidget * widget, gpointer call_data);
+void xsane_scan_dialog(void);
 static void xsane_create_internal_gamma_tables(void);
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
@@ -71,7 +71,7 @@ static int xsane_generate_dummy_filename(int conversion_level)
     free(xsane.dummy_filename);
   }
 
-  if ( (conversion_level == 0) && (xsane.preview->rotation) ) /* scan level with rotation */
+  if ( (conversion_level == 0) && (xsane.scan_rotation) ) /* scan level with rotation */
   {
     tempfile = TRUE;
   }
@@ -85,8 +85,9 @@ static int xsane_generate_dummy_filename(int conversion_level)
        (xsane.xsane_mode == XSANE_FAX) ||
        (xsane.xsane_mode == XSANE_MAIL) ||
        (xsane.xsane_mode == XSANE_VIEWER) ||
-       ( (xsane.xsane_mode == XSANE_SAVE)  && (xsane.xsane_output_format != XSANE_PNM) &&
-         (xsane.xsane_output_format != XSANE_RAW16) && (xsane.xsane_output_format != XSANE_RGBA) ) )
+       ( (xsane.xsane_mode == XSANE_SAVE)  &&
+         (xsane.xsane_output_format != XSANE_PNM) &&
+         (xsane.xsane_output_format != XSANE_RGBA) ) )
   {
     tempfile = TRUE;
   }
@@ -115,15 +116,13 @@ static void xsane_read_image_data(gpointer data, gint source, GdkInputCondition
  SANE_Handle dev = xsane.dev;
  SANE_Status status;
  SANE_Int len;
- int i, j, x;
+ int i, j;
  char buf[255];
 
   DBG(DBG_proc, "xsane_read_image_data\n");
 
   xsane.reading_data = TRUE;
 
-  x = xsane.param.pixels_per_line;
-
   if ( (xsane.param.depth == 1) || (xsane.param.depth == 8) )
   {
    unsigned char buf8[2*32768];
@@ -180,7 +179,19 @@ static void xsane_read_image_data(gpointer data, gint source, GdkInputCondition
 
       if (!len) /* nothing read */
       {
-        break; /* out of data for now, leave while loop */
+        if (xsane.input_tag >= 0)
+        {
+          break; /* leave xsane_read_image_data, will be called by gdk when select_fd event occurs */
+        }
+        else /* no select fd available */
+        {
+          while (gtk_events_pending())
+          {
+            DBG(DBG_info, "calling gtk_main_iteration\n");
+            gtk_main_iteration();
+          }
+          continue; /* we have to keep this loop running because it will never be called again */
+        }
       }
 
       xsane.bytes_read += len;
@@ -247,16 +258,17 @@ static void xsane_read_image_data(gpointer data, gint source, GdkInputCondition
                 {
                   *expanded_buf8ptr = (val & (1 << j)) ? 0x00 : 0xff;
                   expanded_buf8ptr++;
-                  x--;
-                  if (x <= 0)
+
+                  xsane.lineart_to_grayscale_x--;
+                  if (xsane.lineart_to_grayscale_x <= 0)
                   {
-                    x = xsane.param.pixels_per_line;
+                    xsane.lineart_to_grayscale_x = xsane.param.pixels_per_line;
                     break;
                   }
                 }
                 buf8ptr++;
               }
-              fwrite(expanded_buf8, 1, len*8, xsane.out);
+              fwrite(expanded_buf8, 1, (size_t) (expanded_buf8ptr - expanded_buf8), xsane.out);
               free(expanded_buf8);
             }
             else /* save direct to the file */
@@ -309,11 +321,13 @@ static void xsane_read_image_data(gpointer data, gint source, GdkInputCondition
         case SANE_FRAME_GREEN:
         case SANE_FRAME_BLUE:
           {
+           unsigned char rgbbuf[3 * XSANE_3PASS_BUFFER_RGB_SIZE];
+           int pos;
+
             DBG(DBG_info, "3 pass color\n");
 
             if (!xsane.scanner_gamma_color) /* gamma correction by xsane */
             {
-             char val;
              SANE_Int *gamma;
 
               if (xsane.param.format == SANE_FRAME_RED)
@@ -331,19 +345,51 @@ static void xsane_read_image_data(gpointer data, gint source, GdkInputCondition
 
               for (i = 0; i < len; ++i)
               {
-                val = gamma[(int) buf8[i]];
-                fputc(val, xsane.out);
-                fseek(xsane.out, 2, SEEK_CUR);
+                buf8[i] = gamma[(int) buf8[i]];
               }
             }
-            else /* gamma correction by scanner */
+
+            buf8ptr = buf8;
+            pos = 0;
+
+            while(pos < len)
             {
-              for (i = 0; i < len; ++i)
+             int cnt, bytes;
+
+              cnt = len - pos;
+
+              if (cnt > XSANE_3PASS_BUFFER_RGB_SIZE)
               {
-                fputc(buf8[i], xsane.out);
-                fseek(xsane.out, 2, SEEK_CUR);
+                cnt = XSANE_3PASS_BUFFER_RGB_SIZE;
               }
-            }
+
+              bytes = 3 * cnt - 2;
+
+              /* if there already is data: read block of already scanned colors */
+              if( (xsane.param.format > SANE_FRAME_RED) && (cnt > 1) )
+              {
+               long fpos = ftell(xsane.out);
+
+                fseek(xsane.out, 0, SEEK_CUR); /* sync between write and read */
+                fread(rgbbuf, 1, bytes - 1, xsane.out);
+                fseek(xsane.out, fpos, SEEK_SET);
+              }
+
+              /* add just scanned color to block */
+              for(j = 0; j < cnt; j++)
+              {
+                rgbbuf[3 * j] = buf8ptr[j];
+              }
+
+              /* write block back to disk */
+              fwrite(rgbbuf, 1, bytes, xsane.out);
+
+              pos += cnt;
+              buf8ptr += cnt;
+
+              /* skip the bytes for the two other colors */
+              fseek(xsane.out, 2, SEEK_CUR);
+            } /* while(pos < len) */
           }
          break;
 
@@ -490,7 +536,19 @@ static void xsane_read_image_data(gpointer data, gint source, GdkInputCondition
 
       if (!len) /* nothing read */
       {
-        break; /* out of data for now, leave while loop */
+        if (xsane.input_tag >= 0)
+        {
+          break; /* leave xsane_read_image_data, will be called by gdk when select_fd event occurs */
+        }
+        else /* no select fd available */
+        {
+          while (gtk_events_pending())
+          {
+            DBG(DBG_info, "calling gtk_main_iteration\n");
+            gtk_main_iteration();
+          }
+          continue; /* we have to keep this loop running because it will never be called again */
+        }
       }
 
       xsane.bytes_read += len;
@@ -1013,7 +1071,7 @@ void xsane_scan_done(SANE_Status status)
       image_info.threshold        = xsane.threshold;
 
 
-      xsane_write_pnm_header(xsane.out, &image_info);
+      xsane_write_pnm_header(xsane.out, &image_info, 0);
     }
 
     DBG(DBG_info, "closing output file\n");
@@ -1026,7 +1084,7 @@ void xsane_scan_done(SANE_Status status)
   if ( (status == SANE_STATUS_GOOD) || (status == SANE_STATUS_EOF) ) /* no error, do conversion etc. */
   {
     /* do we have to rotate the image ? */
-    if (xsane.preview->rotation)
+    if (xsane.scan_rotation)
     {
      char *old_dummy_filename;
      int abort = 0;
@@ -1070,7 +1128,7 @@ void xsane_scan_done(SANE_Status status)
 
           if (outfile)
           {
-            if (xsane_save_rotate_image(outfile, infile, &image_info, xsane.preview->rotation, xsane.progress_bar, &xsane.cancel_save))
+            if (xsane_save_rotate_image(outfile, infile, &image_info, xsane.scan_rotation, xsane.progress_bar, &xsane.cancel_save))
             {
               abort = 1;
             }
@@ -1149,7 +1207,6 @@ void xsane_scan_done(SANE_Status status)
       }
 
       if ( ( (xsane.xsane_output_format != XSANE_PNM) && /* these files do not need any transformation */
-             (xsane.xsane_output_format != XSANE_RAW16) &&
              (xsane.xsane_output_format != XSANE_RGBA) ) ||
            (xsane.mode == XSANE_GIMP_EXTENSION) )
       { /* ok, we have to do a transformation */
@@ -1243,48 +1300,26 @@ void xsane_scan_done(SANE_Status status)
 
         xsane_read_pnm_header(infile, &image_info);
 
-        imagewidth  = image_info.image_width/(float)printer_resolution; /* width in inch */
-        imageheight = image_info.image_height/(float)printer_resolution; /* height in inch */
+        imagewidth  = 72.0 * image_info.image_width/(float)printer_resolution; /* width in 1/72 inch */
+        imageheight = 72.0 * image_info.image_height/(float)printer_resolution; /* height in 1/72 inch */
 
         memset (&act, 0, sizeof (act)); /* define broken pipe handler */
         act.sa_handler = xsane_sigpipe_handler;
         sigaction (SIGPIPE, &act, 0);
 
+        DBG(DBG_info, "imagewidth  = %f\n 1/72 inch", imagewidth);
+        DBG(DBG_info, "imageheight = %f\n 1/72 inch", imageheight);
 
-        if (preferences.psrotate) /* rotate: landscape */
-        {
-          xsane_save_ps(outfile, infile,
-                        &image_info,
-                        (preferences.printer[preferences.printernr]->bottomoffset +
-                         preferences.printer[preferences.printernr]->height) * 36.0/MM_PER_INCH - imagewidth * 36.0, /* left edge */
-                        (preferences.printer[preferences.printernr]->leftoffset +
-                         preferences.printer[preferences.printernr]->width) * 36.0/MM_PER_INCH - imageheight * 36.0, /* bottom edge */
-                        imagewidth, imageheight,
-                        (preferences.printer[preferences.printernr]->leftoffset +
-                         preferences.printer[preferences.printernr]->width ) * 72.0/MM_PER_INCH,  /* paperwidth */
-                        (preferences.printer[preferences.printernr]->bottomoffset +
-                         preferences.printer[preferences.printernr]->height) * 72.0/MM_PER_INCH, /* paperheight */
-                        1 /* landscape */,
-                        xsane.progress_bar,
-                        &xsane.cancel_save);
-        }
-        else /* do not rotate: portrait */
-        {
-          xsane_save_ps(outfile, infile,
-                        &image_info,
-                        (preferences.printer[preferences.printernr]->leftoffset +
-                         preferences.printer[preferences.printernr]->width) * 36.0/MM_PER_INCH - imagewidth * 36.0, /* left edge */
-                        (preferences.printer[preferences.printernr]->bottomoffset +
-                         preferences.printer[preferences.printernr]->height) * 36.0/MM_PER_INCH - imageheight * 36.0, /* bottom edge */
-                        imagewidth, imageheight,
-                        (preferences.printer[preferences.printernr]->leftoffset +
-                         preferences.printer[preferences.printernr]->width ) * 72.0/MM_PER_INCH,  /* paperwidth */
-                        (preferences.printer[preferences.printernr]->bottomoffset +
-                         preferences.printer[preferences.printernr]->height) * 72.0/MM_PER_INCH, /* paperheight */
-                        0 /* portrait */,
-                        xsane.progress_bar,
-                        &xsane.cancel_save);
-        }
+        xsane_save_ps(outfile, infile,
+                      &image_info,
+                      imagewidth, imageheight,
+                      preferences.printer[preferences.printernr]->leftoffset   * 72.0/MM_PER_INCH, /* paper_left_margin */
+                      preferences.printer[preferences.printernr]->bottomoffset * 72.0/MM_PER_INCH, /* paper_bottom_margin */
+                      preferences.printer[preferences.printernr]->width  * 72.0/MM_PER_INCH, /* usable paperwidth */
+                      preferences.printer[preferences.printernr]->height * 72.0/MM_PER_INCH, /* usable paperheight */
+                      preferences.paper_orientation,
+                      xsane.progress_bar,
+                      &xsane.cancel_save);
       }
       else
       {
@@ -1349,20 +1384,20 @@ void xsane_scan_done(SANE_Status status)
          {
           float imagewidth, imageheight;
 
-           imagewidth  = image_info.image_width/xsane.resolution_x; /* width in inch */
-           imageheight = image_info.image_height/xsane.resolution_y; /* height in inch */
+           imagewidth  = 72.0 * image_info.image_width/xsane.resolution_x; /* width in 1/72 inch */
+           imageheight = 72.0 * image_info.image_height/xsane.resolution_y; /* height in 1/72 inch */
 
-           DBG(DBG_info, "imagewidth  = %f\n", imagewidth);
-           DBG(DBG_info, "imageheight = %f\n", imageheight);
+           DBG(DBG_info, "imagewidth  = %f\n 1/72 inch", imagewidth);
+           DBG(DBG_info, "imageheight = %f\n 1/72 inch", imageheight);
 
            xsane_save_ps(outfile, infile,
                          &image_info,
-                         (preferences.fax_leftoffset   + preferences.fax_width)  * 36.0/MM_PER_INCH - imagewidth * 36.0,
-                         (preferences.fax_bottomoffset + preferences.fax_height) * 36.0/MM_PER_INCH - imageheight * 36.0,
                          imagewidth, imageheight,
-                         (preferences.fax_leftoffset   + preferences.fax_width ) * 72.0/MM_PER_INCH, /* paperwidth */
-                         (preferences.fax_bottomoffset + preferences.fax_height) * 72.0/MM_PER_INCH, /* paperheight */
-                         0 /* portrait */,
+                         preferences.fax_leftoffset   * 72.0/MM_PER_INCH, /* paper_left_margin */
+                         preferences.fax_bottomoffset * 72.0/MM_PER_INCH, /* paper_bottom_margin */
+                         preferences.fax_width  * 72.0/MM_PER_INCH, /* paper_width */
+                         preferences.fax_height * 72.0/MM_PER_INCH, /* paper_height */
+                         0 /* portrait top left */,
                          xsane.progress_bar,
                          &xsane.cancel_save);
            fclose(outfile);
@@ -1531,13 +1566,15 @@ void xsane_scan_done(SANE_Status status)
     xsane.adf_page_counter += 1;
     g_signal_emit_by_name(xsane.start_button, "clicked"); /* press START button */
   }
-  else /* last scan: update histogram */
+  else if ( (!xsane.batch_loop) || ( (status != SANE_STATUS_GOOD) && (status != SANE_STATUS_EOF) ) ) /* last scan: update histogram */
   {
     xsane_set_sensitivity(TRUE);		/* reactivate buttons etc */
     sane_cancel(xsane.dev); /* stop scanning */
     xsane_update_histogram(TRUE /* update raw */);
     xsane_update_param(0);
   }
+
+  xsane.status_of_last_scan = status;
 }
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
@@ -1638,7 +1675,7 @@ static void xsane_start_scan(void)
     default:			frame_type = "unknown"; break;
   }
 
-  if ( (xsane.param.depth == 1) && (xsane.preview->rotation) )
+  if ( (xsane.param.depth == 1) && (xsane.scan_rotation) )
   {
     xsane.expand_lineart_to_grayscale = 1; /* We want to do transformation with lineart scan, so we save it as grayscale */
   }
@@ -1665,7 +1702,8 @@ static void xsane_start_scan(void)
       }
     }
 
-    xsane.out = fopen(xsane.dummy_filename, "wb"); /* b = binary mode for win32 */
+    /* create file: + = also allow read for blocked rgb multiplexing,  b = binary mode for win32 */
+    xsane.out = fopen(xsane.dummy_filename, "wb+");
 
     if (!xsane.out) /* error while opening the dummy_file for writing */
     {
@@ -1706,7 +1744,7 @@ static void xsane_start_scan(void)
  
     image_info.threshold        = xsane.threshold;
 
-    xsane_write_pnm_header(xsane.out, &image_info);
+    xsane_write_pnm_header(xsane.out, &image_info, 0);
 
     fflush(xsane.out);
     xsane.header_size = ftell(xsane.out); /* store header size for 3 pass scan */
@@ -1730,18 +1768,14 @@ static void xsane_start_scan(void)
 
   xsane.input_tag = -1;
 
-#ifndef BUGGY_GDK_INPUT_EXCEPTION
-  /* for unix */
+  xsane.lineart_to_grayscale_x = xsane.param.pixels_per_line;
+
   if ((sane_set_io_mode(dev, SANE_TRUE) == SANE_STATUS_GOOD) && (sane_get_select_fd(dev, &fd) == SANE_STATUS_GOOD))
   {
     DBG(DBG_info, "gdk_input_add\n");
     xsane.input_tag = gdk_input_add(fd, GDK_INPUT_READ | GDK_INPUT_EXCEPTION, xsane_read_image_data, 0);
   }
   else
-#else
-  /* for win32 */
-  sane_set_io_mode(dev, SANE_FALSE);
-#endif
   {
     xsane_read_image_data(0, -1, GDK_INPUT_READ);
   }
@@ -1751,13 +1785,15 @@ static void xsane_start_scan(void)
 
 /* Invoked when the scan button is pressed */
 /* or by scan_done if automatic document feeder is selected */
-void xsane_scan_dialog(GtkWidget * widget, gpointer call_data)
+void xsane_scan_dialog(void)
 {
  char buf[256];
  const SANE_Option_Descriptor *opt;
 
   DBG(DBG_proc, "xsane_scan_dialog\n");
 
+  xsane_set_sensitivity(FALSE);
+
   xsane.reduce_16bit_to_8bit = preferences.reduce_16bit_to_8bit; /* reduce 16 bit image to 8 bit ? */
 
   sane_get_parameters(xsane.dev, &xsane.param); /* update xsane.param */
@@ -1767,6 +1803,10 @@ void xsane_scan_dialog(GtkWidget * widget, gpointer call_data)
     /* correct length of filename counter if it is shorter than minimum length */
     if (!xsane.force_filename)
     {
+      if (xsane.batch_loop) /* we are doing a batch scan, so we need a counter in the filename */
+      {
+        xsane_ensure_counter_in_filename(&preferences.filename, preferences.filename_counter_len);
+      }
       xsane_update_counter_in_filename(&preferences.filename, FALSE, 0, preferences.filename_counter_len);
       xsane_set_filename(preferences.filename); 
     }
@@ -1789,15 +1829,17 @@ void xsane_scan_dialog(GtkWidget * widget, gpointer call_data)
        char buf[256];
 
         fclose(testfile);
+
         snprintf(buf, sizeof(buf), WARN_FILE_EXISTS, xsane.output_filename);
         if (xsane_back_gtk_decision(ERR_HEADER_WARNING, (gchar **) warning_xpm, buf, BUTTON_OVERWRITE, BUTTON_CANCEL, TRUE /* wait */) == FALSE)
         {
+          xsane_set_sensitivity(TRUE);
           return;
         }
       }
     }
 
-    xsane.xsane_output_format = xsane_identify_output_format(xsane.output_filename, &extension);
+    xsane.xsane_output_format = xsane_identify_output_format(xsane.output_filename, preferences.filetype, &extension);
 
     if (xsane.xsane_mode == XSANE_SAVE)
     {
@@ -1812,8 +1854,10 @@ void xsane_scan_dialog(GtkWidget * widget, gpointer call_data)
           snprintf(buf, sizeof(buf), "%s", ERR_NO_OUTPUT_FORMAT);
         }
         xsane_back_gtk_error(buf, TRUE);
-        return;
+        xsane_set_sensitivity(TRUE);
+       return;
       }
+#if 0 /* this is not necessary any more because the saving routines do the conversion */
       else if ( ( ( (xsane.xsane_output_format == XSANE_JPEG) && xsane.param.depth == 16) ||
                   ( (xsane.xsane_output_format == XSANE_PS)   && xsane.param.depth == 16) ) &&
                 ( !xsane.reduce_16bit_to_8bit) )
@@ -1822,17 +1866,20 @@ void xsane_scan_dialog(GtkWidget * widget, gpointer call_data)
         snprintf(buf, sizeof(buf), TEXT_REDUCE_16BIT_TO_8BIT);
         if (xsane_back_gtk_decision(ERR_HEADER_INFO, (gchar **) info_xpm, buf, BUTTON_REDUCE, BUTTON_CANCEL, TRUE /* wait */) == FALSE)
         {
-          return;
+          xsane_set_sensitivity(TRUE);
+         return;
         }
         xsane.reduce_16bit_to_8bit = TRUE;
       }
+#endif
       
 #ifdef SUPPORT_RGBA
       else if ((xsane.xsane_output_format == XSANE_RGBA) && (xsane.param.format != SANE_FRAME_RGBA))
       {
         snprintf(buf, sizeof(buf), "No RGBA data format !!!"); /* user selected output format RGBA, scanner uses other format */
         xsane_back_gtk_error(buf, TRUE);
-        return;
+        xsane_set_sensitivity(TRUE);
+       return;
       }
 #endif
     }
@@ -1841,7 +1888,8 @@ void xsane_scan_dialog(GtkWidget * widget, gpointer call_data)
     {
       snprintf(buf, sizeof(buf), "Special format RGBA only supported in scan mode !!!");
       xsane_back_gtk_error(buf, TRUE);
-      return;
+      xsane_set_sensitivity(TRUE);
+     return;
     }
 #endif
 
@@ -1852,7 +1900,8 @@ void xsane_scan_dialog(GtkWidget * widget, gpointer call_data)
       {
         snprintf(buf, sizeof(buf), "Image data of type SANE_FRAME_RGBA\ncan only be saved in rgba or png format");
         xsane_back_gtk_error(buf, TRUE);
-        return;
+        xsane_set_sensitivity(TRUE);
+       return;
       }
     }
 #endif
@@ -1879,7 +1928,8 @@ void xsane_scan_dialog(GtkWidget * widget, gpointer call_data)
           snprintf(buf, sizeof(buf), TEXT_GIMP_REDUCE_16BIT_TO_8BIT);
           if (xsane_back_gtk_decision(ERR_HEADER_INFO, (gchar **) info_xpm, buf, BUTTON_REDUCE, BUTTON_CANCEL, TRUE /* wait */) == FALSE)
           {
-            return;
+            xsane_set_sensitivity(TRUE);
+           return;
           }
           xsane.reduce_16bit_to_8bit = TRUE;
         }
@@ -1887,6 +1937,7 @@ void xsane_scan_dialog(GtkWidget * widget, gpointer call_data)
         {
           snprintf(buf, sizeof(buf), ERR_GIMP_BAD_DEPTH, xsane.param.depth);
           xsane_back_gtk_error(buf, TRUE);
+          xsane_set_sensitivity(TRUE);
          return;
         }
       }
@@ -2036,9 +2087,9 @@ void xsane_scan_dialog(GtkWidget * widget, gpointer call_data)
     xsane.gamma_data = 0;
   }
 
-  xsane_clear_histogram(&xsane.histogram_raw);
-  xsane_clear_histogram(&xsane.histogram_enh);    
-  xsane_set_sensitivity(FALSE);
+//  xsane_clear_histogram(&xsane.histogram_raw);
+//  xsane_clear_histogram(&xsane.histogram_enh);    
+//  xsane_set_sensitivity(FALSE);
 
   while (gtk_events_pending())
   {
