@@ -29,7 +29,7 @@
 #define XSANE_LANGUAGE_DIR		_("language_dir")
 
 #define XSANE_STRSTATUS(status)		_(sane_strstatus(status))
-#define _BGT(text)			dgettext(xsane.backend, text)
+#define _BGT(text)			dgettext(xsane.backend_translation, text)
 
 #define XSANE_COPYRIGHT_SIGN		_("(c)") /* can be translated with \251 */
 
@@ -39,7 +39,10 @@
 #define WINDOW_GPL			_("GPL - the license")
 #define WINDOW_EULA			_("End User License Agreement")
 #define WINDOW_INFO			_("info")
+#define WINDOW_LOAD_BATCH_LIST		_("load batch list")
+#define WINDOW_SAVE_BATCH_LIST		_("save batch list")
 #define WINDOW_BATCH_SCAN		_("batch scan")
+#define WINDOW_BATCH_RENAME		_("rename batch area")
 #define WINDOW_FAX_PROJECT		_("fax project")
 #define WINDOW_FAX_RENAME		_("rename fax page")
 #define WINDOW_FAX_INSERT		_("insert ps-file into fax")
@@ -66,6 +69,7 @@
 #define WINDOW_DESPECKLE		_("Despeckle image")
 #define WINDOW_BLUR			_("Blur image")
 #define WINDOW_STORE_MEDIUM		_("Store medium definition")
+#define WINDOW_NO_DEVICES		_("No devices available")
 
 #define MENU_FILE			_("File")
 #define MENU_PREFERENCES		_("Preferences")
@@ -90,7 +94,7 @@
 #define FRAME_RAW_IMAGE			_("Raw image")
 #define FRAME_ENHANCED_IMAGE		_("Enhanced image")
 
-#define BUTTON_START			_("Start")
+#define BUTTON_SCAN			_("Scan")
 #define BUTTON_OK			_("Ok")
 #define BUTTON_ACCEPT			_("Accept")
 #define BUTTON_NOT_ACCEPT		_("Not accept")
@@ -100,8 +104,10 @@
 #define BUTTON_CONT_AT_OWN_RISK		_("Continue at your own risk")
 #define BUTTON_BROWSE			_("Browse")
 #define BUTTON_CLOSE			_("Close")
+#define BUTTON_HELP			_("Help")
 #define BUTTON_OVERWRITE		_("Overwrite")
-#define BUTTON_ADD_AREA			_("Add area")
+#define BUTTON_BATCH_LIST_SCAN		_("Scan batch list")
+#define BUTTON_BATCH_AREA_SCAN		_("Scan selected area")
 #define BUTTON_PAGE_DELETE		_("Delete page")
 #define BUTTON_PAGE_SHOW		_("Show page")
 #define BUTTON_PAGE_RENAME		_("Rename page")
@@ -126,6 +132,7 @@
 #define RADIO_BUTTON_SAVE_DEVPREFS_AT_EXIT _("Save device preferences at exit")
 #define RADIO_BUTTON_OVERWRITE_WARNING	_("Overwrite warning")
 #define RADIO_BUTTON_SKIP_EXISTING_NRS	_("Skip existing filenames")
+#define RADIO_BUTTON_SAVE_PNM16_AS_ASCII _("Save 16bit pnm in ascii format")
 #define RADIO_BUTTON_REDUCE_16BIT_TO_8BIT _("Reduce 16 bit image to 8 bit")
 #define RADIO_BUTTON_WINDOW_FIXED	_("Main window size fixed")
 #define RADIO_BUTTON_DISABLE_GIMP_PREVIEW_GAMMA	_("Disable gimp preview gamma")
@@ -221,6 +228,14 @@
 #define TEXT_VIEWER_IMAGE_INFO		_("Size %d x %d pixel, %d bit/color, %d colors, %1.0f dpi x %1.0f dpi, %1.1f %s")
 #define TEXT_DESPECKLE_RADIUS		_("Despeckle radius:")
 #define TEXT_BLUR_RADIUS		_("Blur radius:")
+#define TEXT_BATCH_AREA_DEFAULT_NAME	_("(no name)")
+#define TEXT_BATCH_LIST_AREANAME	_("Area name:")
+#define TEXT_BATCH_LIST_SCANMODE	_("Scanmode:")
+#define TEXT_BATCH_LIST_GEOMETRY_TL	_("Top left:")
+#define TEXT_BATCH_LIST_GEOMETRY_SIZE	_("Size:")
+#define TEXT_BATCH_LIST_RESOLUTION	_("Resolution:")
+#define TEXT_BATCH_LIST_BIT_DEPTH	_("Bit depth:")
+#define TEXT_BATCH_LIST_BY_GUI		_("as selected")
 
 #define TEXT_SETUP_PRINTER_SEL		_("Printer selection:")
 #define TEXT_SETUP_PRINTER_NAME		_("Name:")
@@ -266,6 +281,7 @@
 #define TEXT_SETUP_FAX_POSTSCRIPT_OPT	_("Postscriptfile option:")
 #define TEXT_SETUP_FAX_NORMAL_MODE_OPT	_("Normal mode option:")
 #define TEXT_SETUP_FAX_FINE_MODE_OPT	_("Fine mode option:")
+#define TEXT_SETUP_FAX_PROGRAM_DEFAULTS _("Set program defaults for:")
 #define TEXT_SETUP_FAX_VIEWER		_("Viewer (Postscript):")
 #define TEXT_SETUP_FAX_WIDTH		_("Width")
 #define TEXT_SETUP_FAX_HEIGHT		_("Height")
@@ -309,6 +325,7 @@
 #define MENU_ITEM_SHOW_PREVIEW		_("Show preview")
 #define MENU_ITEM_SHOW_HISTOGRAM	_("Show histogram")
 #define MENU_ITEM_SHOW_GAMMA		_("Show gamma curve")
+#define MENU_ITEM_SHOW_BATCH_SCAN	_("Show batch scan")
 #define MENU_ITEM_SHOW_STANDARDOPTIONS	_("Show standard options")
 #define MENU_ITEM_SHOW_ADVANCEDOPTIONS	_("Show advanced options")
 
@@ -359,8 +376,8 @@
 #define MENU_ITEM_PRESET_AREA_ADD_SEL	_("Add selection to list")
 #define MENU_ITEM_PRESET_AREA_RENAME	_("Rename definiton")
 #define MENU_ITEM_PRESET_AREA_DELETE	_("Delete definition")
-#define MENU_OTEM_PRESET_AREA_MOVE_UP	_("Move up")
-#define MENU_OTEM_PRESET_AREA_MOVE_DWN	_("Move down")
+#define MENU_ITEM_PRESET_AREA_MOVE_UP	_("Move up")
+#define MENU_ITEM_PRESET_AREA_MOVE_DWN	_("Move down")
 
 #define PROGRESS_SCANNING		_("Scanning")
 #define PROGRESS_RECEIVING_FRAME_DATA	_("Receiving %s data")
@@ -375,6 +392,7 @@
 #define PROGRESS_SCALING_DATA		_("Scaling image")
 #define PROGRESS_DESPECKLING_DATA	_("Despeckling image")
 #define PROGRESS_BLURING_DATA		_("Bluring image")
+#define PROGRESS_OCR			_("OCR in progress")
 
 #define DESC_SCAN_START			_("Start scan <Ctrl-Enter>")
 #define DESC_SCAN_CANCEL		_("Cancel scan <ESC>")
@@ -473,6 +491,10 @@
 #define DESC_SAVE_DEVPREFS_AT_EXIT	_("Save device dependant preferences in default file at exit of xsane")
 #define DESC_OVERWRITE_WARNING		_("Warn before overwriting an existing file")
 #define DESC_SKIP_EXISTING		_("If filename counter is automatically increased, used numbers are skipped")
+#define DESC_SAVE_PNM16_AS_ASCII	_("When a 16 bit image shall be saved in pnm format then use ascii format " \
+                                          "instead of binary format. The binary format is a new format that is not " \
+                                          "supported by all programs. The ascii format is supported by more programs " \
+                                          "but it produces really huge files!!!")
 #define DESC_REDUCE_16BIT_TO_8BIT	_("If scanner sends image with 16 bits/color save image with 8 bits/color")
 #define DESC_PSFILE_WIDTH		_("Width of paper for postscript files")
 #define DESC_PSFILE_HEIGHT		_("Height of paper for postscript files")
@@ -536,6 +558,7 @@
 #define DESC_PERMISSION_WRITE		_("write")
 #define DESC_PERMISSION_EXECUTE		_("execute")
 
+#define DESC_ADD_BATCH			_("Add selection for batch scan")
 #define DESC_PIPETTE_WHITE		_("Pick white point")
 #define DESC_PIPETTE_GRAY		_("Pick gray point")
 #define DESC_PIPETTE_BLACK		_("Pick black point")
@@ -553,6 +576,8 @@
 
 #define DESC_PRESET_AREA		_("Preset area")
 #define DESC_ROTATION			_("Rotate preview and scan")
+#define DESC_RATIO			_("Aspect ratio of selection")
+#define DESC_PAPER_ORIENTATION		_("Define image position for printing")
 
 #define DESC_VIEWER_SAVE		_("Save image")
 #define DESC_VIEWER_OCR			_("Optical Character Recognition")
@@ -560,17 +585,25 @@
 #define DESC_VIEWER_SCALE		_("Scale image")
 #define DESC_VIEWER_DESPECKLE		_("Despeckle image")
 #define DESC_VIEWER_BLUR		_("Blur image")
-#define DESC_VIEWER_ROTATE90		_("Rotate image 90 degree")
-#define DESC_VIEWER_ROTATE180		_("Rotate image 180 degree")
-#define DESC_VIEWER_ROTATE270		_("Rotate image 270 degree")
-#define DESC_VIEWER_MIRROR_X		_("Mirror image at vertical axis")
-#define DESC_VIEWER_MIRROR_Y		_("Mirror image at horizontal axis")
+#define DESC_ROTATE90			_("Rotate image 90 degree")
+#define DESC_ROTATE180			_("Rotate image 180 degree")
+#define DESC_ROTATE270			_("Rotate image 270 degree")
+#define DESC_MIRROR_X			_("Mirror image at vertical axis")
+#define DESC_MIRROR_Y			_("Mirror image at horizontal axis")
 #define DESC_VIEWER_ZOOM		_("Zoom image")
 #define DESC_STORE_MEDIUM		_("Store medium")
 #define DESC_DELETE_MEDIUM		_("Delete active medium")
 #define DESC_SCALE_FACTOR		_("Scale factor")
 #define DESC_X_SCALE_FACTOR		_("X-Scale factor")
 #define DESC_Y_SCALE_FACTOR		_("Y-Scale factor")
+#define DESC_SCALE_WIDTH		_("Scale image to width [in pixel]")
+#define DESC_SCALE_HEIGHT		_("Scale image to height [in pixel]")
+#define DESC_BATCH_LIST_EMPTY		_("Empty batch list")
+#define DESC_BATCH_LIST_SAVE		_("Save batch list")
+#define DESC_BATCH_LIST_LOAD		_("Load batch list")
+#define DESC_BATCH_RENAME		_("Rename area")
+#define DESC_BATCH_ADD			_("Add selected preview area to batch list")
+#define DESC_BATCH_DEL			_("Delete selected area from batch list")
 
 #define ERR_HOME_DIR			_("Failed to determine home directory:")
 #define ERR_CHANGE_WORKING_DIR		_("Failed to change working directory to")
@@ -634,6 +667,7 @@ YOU ARE ALONE!\
 #define ERR_HEADER_ERROR		_("Error")
 #define ERR_HEADER_WARNING		_("Warning")
 #define ERR_HEADER_INFO			_("Information")
+#define ERR_HEADER_CHILD_PROCESS_ERROR	_("Child process error")
 
 #define ERR_FAILED_CREATE_FILE		_("Failed to create file:")
 #define ERR_LOAD_DEVICE_SETTINGS	_("Error while loading device settings:")
@@ -697,6 +731,14 @@ The format of [DEVICE] is backendname:devicefile (e.g. umax:/dev/scanner).\n\
 #define XSANE_GIMP_MENU_DIALOG_OLD	_("<Toolbox>/Xtns/XSane/Device dialog...")
 #define XSANE_GIMP_MENU_OLD		_("<Toolbox>/Xtns/XSane/")
 
+#define HELP_NO_DEVICES	      _("Possible reasons:\n" \
+				"1) There really is no device that is supported by SANE\n" \
+				"2) Supported devices are busy\n" \
+				"3) The permissions for the device file do not allow you to use it - try as root\n" \
+				"4) The backend is not loaded by SANE (man sane-dll)\n" \
+				"5) The backend is not configured correct (man sane-\"backendname\")\n" \
+				"6) Possibly there is more than one SANE version installed" )
+
 /* strings that are used in structures, so it is not allowed to use _()/gettext() here */
 /* gettext_noop does mark these texts but does not change the string */
 
@@ -718,7 +760,7 @@ The format of [DEVICE] is backendname:devicefile (e.g. umax:/dev/scanner).\n\
 #define MENU_ITEM_SURFACE_letter_P		N_("letter port.")
 #define MENU_ITEM_SURFACE_letter_L		N_("letter land.")
 
-#define MENU_ITEM_MEDIUM_FULL_RANGE		N_("Full range")
+#define MENU_ITEM_MEDIUM_FULL_COLOR_RANGE	N_("Full color range")
 #define MENU_ITEM_MEDIUM_SLIDE			N_("Slide")
 #define MENU_ITEM_MEDIUM_STANDARD_NEG		N_("Standard negative")
 #define MENU_ITEM_MEDIUM_AGFA_NEG		N_("Agfa negative")
