@@ -23,7 +23,6 @@
 /* ---------------------------------------------------------------------------------------------------------------------- */
  
 #include "xsane.h"
-/* #include <sys/param.h> */
 #include "xsane-back-gtk.h"
 #include "xsane-front-gtk.h"
 #include "xsane-preferences.h"
@@ -42,7 +41,6 @@
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
 static int xsane_viewer_zoom[] = {35, 50, 71, 100, 141, 200, 282, 400 };
-#define XSANE_VIEWER_ZOOM_ITEMS 8
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
@@ -138,6 +136,7 @@ static void xsane_viewer_save_callback(GtkWidget *window, gpointer data)
  char *inputfilename;
  char windowname[256];
  int output_format;
+ char *filetype = NULL;
 
   DBG(DBG_proc, "xsane_viewer_save_callback\n");
 
@@ -151,12 +150,12 @@ static void xsane_viewer_save_callback(GtkWidget *window, gpointer data)
   {
    int abort = 0;
 
-    strcpy(outputfilename, preferences.filename);
+    strncpy(outputfilename, preferences.filename, sizeof(outputfilename));
  
     snprintf(windowname, sizeof(windowname), "%s %s %s", xsane.prog_name, WINDOW_VIEWER_OUTPUT_FILENAME, xsane.device_text);
  
     umask((mode_t) preferences.directory_umask); /* define new file permissions */
-    abort = xsane_back_gtk_get_filename(windowname, outputfilename, sizeof(outputfilename), outputfilename, TRUE, TRUE, FALSE);
+    abort = xsane_back_gtk_get_filename(windowname, outputfilename, sizeof(outputfilename), outputfilename, &filetype, TRUE, TRUE, FALSE, TRUE);
     umask(XSANE_DEFAULT_UMASK); /* define new file permissions */ 
 
     if (abort)
@@ -187,7 +186,7 @@ static void xsane_viewer_save_callback(GtkWidget *window, gpointer data)
 
   inputfilename = strdup(v->filename);
 
-  output_format = xsane_identify_output_format(outputfilename, 0);
+  output_format = xsane_identify_output_format(outputfilename, filetype, 0);
 
   if (v->reduce_to_lineart) /* reduce grayscale image to lineart before saving */
   {
@@ -250,14 +249,7 @@ static void xsane_viewer_ocr_callback(GtkWidget *window, gpointer data)
  char outputfilename[1024];
  char *extensionptr;
  char windowname[256];
- char *arg[1000];
- char buf[256];
- int argnr;
- pid_t pid;
  int abort = 0;
- int i;
- int pipefd[2]; /* for progress communication with gocr */
- FILE *ocr_progress = NULL;
 
   DBG(DBG_proc, "xsane_viewer_ocr_callback\n");
 
@@ -275,7 +267,7 @@ static void xsane_viewer_ocr_callback(GtkWidget *window, gpointer data)
   snprintf(windowname, sizeof(windowname), "%s %s %s", xsane.prog_name, WINDOW_OCR_OUTPUT_FILENAME, xsane.device_text);
  
   umask((mode_t) preferences.directory_umask); /* define new file permissions */
-  abort = xsane_back_gtk_get_filename(windowname, outputfilename, sizeof(outputfilename), outputfilename, TRUE, TRUE, FALSE);
+  abort = xsane_back_gtk_get_filename(windowname, outputfilename, sizeof(outputfilename), outputfilename, NULL, TRUE, TRUE, FALSE, FALSE);
   umask(XSANE_DEFAULT_UMASK); /* define new file permissions */ 
 
   if (abort)
@@ -289,132 +281,9 @@ static void xsane_viewer_ocr_callback(GtkWidget *window, gpointer data)
     gtk_main_iteration();
   }
 
-  argnr = xsane_parse_options(preferences.ocr_command, arg);
-
-  arg[argnr++] = strdup(preferences.ocr_inputfile_option);
-  arg[argnr++] = strdup(v->filename);
-
-  arg[argnr++] = strdup(preferences.ocr_outputfile_option);
-  arg[argnr++] = strdup(outputfilename);
-
-  if (preferences.ocr_use_gui_pipe)
-  {
-    if (!pipe(pipefd)) /* success */
-    {
-      DBG(DBG_info, "created pipe for progress communication\n");
-
-      arg[argnr++] = strdup(preferences.ocr_gui_outfd_option);
-
-      snprintf(buf, sizeof(buf),"%d", pipefd[1]);
-      arg[argnr++] = strdup(buf);
-    }
-    else
-    {
-      DBG(DBG_info, "could not create pipe for progress communication\n");
-      pipefd[0] = 0;
-      pipefd[1] = 0;
-    }
-  }
-  else
-  {
-    DBG(DBG_info, "no pipe for progress communication requested\n");
-    pipefd[0] = 0;
-    pipefd[1] = 0;
-  }
-
-  arg[argnr] = 0;
-
-  pid = fork(); 
-
-  if (pid == 0) /* new process */
-  {
-    if (pipefd[0]) /* did we create the progress pipe? */
-    {
-      close(pipefd[0]); /* close reading end of pipe */
-    }
-
-    DBG(DBG_info, "trying to change user id fo new subprocess:\n");
-    DBG(DBG_info, "old effective uid = %d\n", geteuid());
-    setuid(getuid());
-    DBG(DBG_info, "new effective uid = %d\n", geteuid());
-
-
-    execvp(arg[0], arg); /* does not return if successfully */
-    DBG(DBG_error, "%s %s\n", ERR_FAILED_EXEC_OCR_CMD, preferences.ocr_command);
-    _exit(0); /* do not use exit() here! otherwise gtk gets in trouble */
-  }
-
-  if (pipefd[1])
-  {
-    close(pipefd[1]); /* close writing end of pipe */
-    ocr_progress = fdopen(pipefd[0], "r"); /* open reading end of pipe as file */
-  }
-
-  for (i=0; i<argnr; i++)
-  {
-    free(arg[i]);
-  }
-
-  if (ocr_progress) /* pipe available */
-  {
-    gtk_progress_set_format_string(GTK_PROGRESS(v->progress_bar), "OCR in progress");
-    gtk_progress_bar_update(GTK_PROGRESS_BAR(v->progress_bar), 0.0);
-
-    while (!feof(ocr_progress))
-    {
-     int progress;
-      fgets(buf, sizeof(buf), ocr_progress);
-
-      if (!strncmp(preferences.ocr_progress_keyword, buf, strlen(preferences.ocr_progress_keyword)))
-      {
-        sscanf(buf + strlen(preferences.ocr_progress_keyword), "%d", &progress);
-        if (progress < 0)
-        {
-          progress = 0;
-        }
-
-        if (progress > 100)
-        {
-          progress = 100;
-        }
-
-        gtk_progress_bar_update(GTK_PROGRESS_BAR(v->progress_bar), progress/100.0);
-      }
-
-      while (gtk_events_pending())
-      {
-        gtk_main_iteration();
-      }
-    }
-
-    gtk_progress_set_format_string(GTK_PROGRESS(v->progress_bar), "");
-    gtk_progress_bar_update(GTK_PROGRESS_BAR(v->progress_bar), 0.0);
-  }
-  else /* no pipe available */
-  {
-    while (pid)
-    {
-     int status = 0;
-     pid_t pid_status = waitpid(pid, &status, WNOHANG);
- 
-      if (pid == pid_status)
-      {
-        pid = 0; /* ok, child process has terminated */
-      }
- 
-      while (gtk_events_pending())
-      {
-        gtk_main_iteration();
-      }
-    } 
-  }
+  xsane_save_image_as_text(v->filename, outputfilename, v->progress_bar, &v->cancel_save);
 
   gtk_widget_set_sensitive(GTK_WIDGET(v->button_box), TRUE);
-
-  if (pipefd[0])
-  {
-    fclose(ocr_progress); /* close reading end of pipe */
-  }
 }
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
@@ -473,44 +342,100 @@ static void xsane_viewer_clone_callback(GtkWidget *window, gpointer data)
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
-static void xsane_viewer_range_changed(GtkAdjustment *adj_data, double *val)
+static void xsane_viewer_adjustment_float_changed(GtkAdjustment *adj_data, float *val)
 {
-   *val = adj_data->value;
+  *val = (float) adj_data->value;
 }
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
-static void xsane_viewer_spinbutton_float_changed(GtkWidget *spinbutton, gpointer data)
+static void xsane_viewer_adjustment_int_changed(GtkAdjustment *adj_data, int *val)
 {
- float *val = (float *) data;
-
-#ifdef HAVE_GTK2
-  *val = (float) gtk_spin_button_get_value((GtkSpinButton *) spinbutton);
-#else
-  *val = gtk_spin_button_get_value_as_float((GtkSpinButton *) spinbutton);
-#endif
+  *val = (int) adj_data->value;
 }
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
-static void xsane_viewer_spinbutton_int_changed(GtkWidget *spinbutton, gpointer data)
+static void xsane_viewer_button_changed(GtkWidget *button, int *val)
 {
- int *val = (int *) data;
-
-  *val = gtk_spin_button_get_value_as_int((GtkSpinButton *) spinbutton);
-}
-
-/* ---------------------------------------------------------------------------------------------------------------------- */
-
-static void xsane_viewer_button_changed(GtkWidget *button, gpointer data)
-{
- int *val = (int *) data;
-
   *val = GTK_TOGGLE_BUTTON(button)->active;
 }
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
+static void xsane_viewer_scale_set_scale_value_and_adjustments(GtkAdjustment *adj_data, double *scale_val)
+{
+ GtkAdjustment *adj;
+ int image_width, image_height;
+
+  *scale_val = adj_data->value;
+
+  image_width  = (int) gtk_object_get_data(GTK_OBJECT(adj_data), "image_width");
+  image_height = (int) gtk_object_get_data(GTK_OBJECT(adj_data), "image_height");
+
+  adj = (GtkAdjustment*) gtk_object_get_data(GTK_OBJECT(adj_data), "size-x-adjustment");
+  if ((adj) && (image_width))
+  {
+    gtk_adjustment_set_value(adj, (*scale_val) * image_width);
+  }
+
+  adj = (GtkAdjustment*) gtk_object_get_data(GTK_OBJECT(adj_data), "size-y-adjustment");
+  if ((adj) && (image_height))
+  {
+    gtk_adjustment_set_value(adj, (*scale_val) * image_height);
+  }
+}
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
+static void xsane_viewer_scale_set_size_x_value_and_adjustments(GtkAdjustment *adj_data, double *scale_val)
+{
+ GtkAdjustment *adj;
+ int image_width, image_height;
+
+  image_width  = (int) gtk_object_get_data(GTK_OBJECT(adj_data), "image_width");
+  image_height = (int) gtk_object_get_data(GTK_OBJECT(adj_data), "image_height");
+
+  if (!image_width)
+  {
+    return; /* we are not able to calulate the scale value */
+  }
+
+  *scale_val = adj_data->value / image_width;
+
+  adj = (GtkAdjustment*) gtk_object_get_data(GTK_OBJECT(adj_data), "scale-adjustment");
+  if (adj)
+  {
+    gtk_adjustment_set_value(adj, *scale_val);
+  }
+}
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
+static void xsane_viewer_scale_set_size_y_value_and_adjustments(GtkAdjustment *adj_data, double *scale_val)
+{
+ GtkAdjustment *adj;
+ int image_width, image_height;
+
+  image_width  = (int) gtk_object_get_data(GTK_OBJECT(adj_data), "image_width");
+  image_height = (int) gtk_object_get_data(GTK_OBJECT(adj_data), "image_height");
+
+  if (!image_height)
+  {
+    return; /* we are not able to calulate the scale value */
+  }
+
+  *scale_val = adj_data->value / image_height;
+
+  adj = (GtkAdjustment*) gtk_object_get_data(GTK_OBJECT(adj_data), "scale-adjustment");
+  if (adj)
+  {
+    gtk_adjustment_set_value(adj, *scale_val);
+  }
+}
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
 static void xsane_viewer_scale_callback(GtkWidget *window, gpointer data)
 {
  Viewer *v = (Viewer *) data;
@@ -519,12 +444,34 @@ static void xsane_viewer_scale_callback(GtkWidget *window, gpointer data)
  GtkWidget *hbox, *vbox;
  GtkWidget *button;
  GtkObject *scale_widget, *scalex_widget, *scaley_widget;
+ GtkAdjustment *adjustment_size_x;
+ GtkAdjustment *adjustment_size_y;
+ GtkWidget *spinbutton;
+ GdkPixmap *pixmap;
+ GdkBitmap *mask;
+ GtkWidget *pixmapwidget;
  char buf[256];
+ FILE *infile;
+ Image_info image_info;
 
   DBG(DBG_proc, "xsane_viewer_scale_callback\n");
 
   gtk_widget_set_sensitive(GTK_WIDGET(v->button_box), FALSE);
 
+  infile = fopen(v->filename, "rb");
+  if (!infile)
+  {
+    DBG(DBG_error, "could not load file %s\n", v->filename);
+    gtk_widget_set_sensitive(GTK_WIDGET(v->button_box), TRUE);
+
+   return;
+  }
+
+  xsane_read_pnm_header(infile, &image_info);
+
+  fclose(infile);
+
+
   if (v->output_filename)
   {
     snprintf(buf, sizeof(buf), "%s: %s", WINDOW_SCALE, v->output_filename); 
@@ -579,28 +526,141 @@ static void xsane_viewer_scale_callback(GtkWidget *window, gpointer data)
 
   if (v->bind_scale)
   {
+    hbox = gtk_hbox_new(FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(hbox), 4); 
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
+    gtk_widget_show(hbox);
+
     /* scale factor: <-> */
-    xsane_range_new_with_pixmap(xsane.xsane_window->window, GTK_BOX(vbox), zoom_xpm, 
+    xsane_range_new_with_pixmap(xsane.xsane_window->window, GTK_BOX(hbox), zoom_xpm, 
                                 DESC_SCALE_FACTOR,
-                                0.01, 10.0, 0.01, 0.1, 2, &v->x_scale_factor, &scale_widget,
-                                0, xsane_viewer_range_changed,
+                                0.01, 4.0, 0.01, 0.1, 2, &v->x_scale_factor, &scale_widget,
+                                0, xsane_viewer_scale_set_scale_value_and_adjustments,
                                 TRUE);
+
+    /* x-size */
+    pixmap = gdk_pixmap_create_from_xpm_d(selection_dialog->window, &mask, xsane.bg_trans, (gchar **) size_x_xpm);
+    pixmapwidget = gtk_image_new_from_pixmap(pixmap, mask);
+    gtk_box_pack_start(GTK_BOX(hbox), pixmapwidget, FALSE, FALSE, 20);
+    gtk_widget_show(pixmapwidget);
+    gdk_drawable_unref(pixmap);
+
+    adjustment_size_x = (GtkAdjustment *) gtk_adjustment_new(v->x_scale_factor * image_info.image_width , 0.01 * image_info.image_width, 4.0 * image_info.image_width, 1.0, 5.0, 0.0);
+    spinbutton = gtk_spin_button_new(adjustment_size_x, 0, 0);
+    g_signal_connect(GTK_OBJECT(adjustment_size_x), "value_changed", (GtkSignalFunc) xsane_viewer_scale_set_size_x_value_and_adjustments, (void *) &v->x_scale_factor);
+    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spinbutton), FALSE);
+    gtk_widget_set_size_request(spinbutton, 80, -1);
+    gtk_box_pack_start(GTK_BOX(hbox), spinbutton, FALSE, FALSE, 0);
+    gtk_widget_show(spinbutton);
+    xsane_back_gtk_set_tooltip(xsane.tooltips, spinbutton, DESC_SCALE_WIDTH); 
+
+    /* y-size */
+    pixmap = gdk_pixmap_create_from_xpm_d(selection_dialog->window, &mask, xsane.bg_trans, (gchar **) size_y_xpm);
+    pixmapwidget = gtk_image_new_from_pixmap(pixmap, mask);
+    gtk_box_pack_start(GTK_BOX(hbox), pixmapwidget, FALSE, FALSE, 20);
+    gtk_widget_show(pixmapwidget);
+    gdk_drawable_unref(pixmap);
+
+    adjustment_size_y = (GtkAdjustment *) gtk_adjustment_new(v->x_scale_factor * image_info.image_height , 0.01 * image_info.image_height, 4.0 * image_info.image_height, 1.0, 5.0, 0.0);
+    spinbutton = gtk_spin_button_new(adjustment_size_y, 0, 0);
+    g_signal_connect(GTK_OBJECT(adjustment_size_y), "value_changed", (GtkSignalFunc) xsane_viewer_scale_set_size_y_value_and_adjustments, (void *) &v->x_scale_factor);
+    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spinbutton), FALSE);
+    gtk_widget_set_size_request(spinbutton, 80, -1);
+    gtk_box_pack_start(GTK_BOX(hbox), spinbutton, FALSE, FALSE, 0);
+    gtk_widget_show(spinbutton);
+    xsane_back_gtk_set_tooltip(xsane.tooltips, spinbutton, DESC_SCALE_HEIGHT); 
+
+    gtk_object_set_data(GTK_OBJECT(scale_widget), "size-x-adjustment", (void *) adjustment_size_x);
+    gtk_object_set_data(GTK_OBJECT(scale_widget), "size-y-adjustment", (void *) adjustment_size_y);
+    gtk_object_set_data(GTK_OBJECT(scale_widget), "image_width",       (void *) image_info.image_width);
+    gtk_object_set_data(GTK_OBJECT(scale_widget), "image_height",      (void *) image_info.image_height);
+
+    gtk_object_set_data(GTK_OBJECT(adjustment_size_x), "scale-adjustment",   (void *) scale_widget);
+    gtk_object_set_data(GTK_OBJECT(adjustment_size_x), "size-y-adjustment",  (void *) adjustment_size_y);
+    gtk_object_set_data(GTK_OBJECT(adjustment_size_x), "image_width",        (void *) image_info.image_width);
+    gtk_object_set_data(GTK_OBJECT(adjustment_size_x), "image_height",       (void *) image_info.image_height);
+
+    gtk_object_set_data(GTK_OBJECT(adjustment_size_y), "scale-adjustment",   (void *) scale_widget);
+    gtk_object_set_data(GTK_OBJECT(adjustment_size_y), "size-x-adjustment",  (void *) adjustment_size_x);
+    gtk_object_set_data(GTK_OBJECT(adjustment_size_y), "image_width",        (void *) image_info.image_width);
+    gtk_object_set_data(GTK_OBJECT(adjustment_size_y), "image_height",       (void *) image_info.image_height);
   }
   else
   {
+    /* X */
+    hbox = gtk_hbox_new(FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(hbox), 4); 
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
+    gtk_widget_show(hbox);
+
     /* x_scale factor: <-> */
-    xsane_range_new_with_pixmap(xsane.xsane_window->window, GTK_BOX(vbox), zoom_x_xpm,
+    xsane_range_new_with_pixmap(xsane.xsane_window->window, GTK_BOX(hbox), zoom_x_xpm,
                                 DESC_X_SCALE_FACTOR,
-                                0.01, 10.0, 0.01, 0.1, 2, &v->x_scale_factor, &scalex_widget,
-                                0, xsane_viewer_range_changed,
+                                0.01, 4.0, 0.01, 0.1, 2, &v->x_scale_factor, &scalex_widget,
+                                0, xsane_viewer_scale_set_scale_value_and_adjustments,
                                 TRUE);
 
+    /* x-size */
+    pixmap = gdk_pixmap_create_from_xpm_d(selection_dialog->window, &mask, xsane.bg_trans, (gchar **) size_x_xpm);
+    pixmapwidget = gtk_image_new_from_pixmap(pixmap, mask);
+    gtk_box_pack_start(GTK_BOX(hbox), pixmapwidget, FALSE, FALSE, 20);
+    gtk_widget_show(pixmapwidget);
+    gdk_drawable_unref(pixmap);
+
+    adjustment_size_x = (GtkAdjustment *) gtk_adjustment_new(v->x_scale_factor * image_info.image_width , 0.01 * image_info.image_width, 4.0 * image_info.image_width, 1.0, 5.0, 0.0);
+    spinbutton = gtk_spin_button_new(adjustment_size_x, 0, 0);
+    g_signal_connect(GTK_OBJECT(adjustment_size_x), "value_changed", (GtkSignalFunc) xsane_viewer_scale_set_size_x_value_and_adjustments, (void *) &v->x_scale_factor);
+    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spinbutton), FALSE);
+    gtk_widget_set_size_request(spinbutton, 80, -1);
+    gtk_box_pack_start(GTK_BOX(hbox), spinbutton, FALSE, FALSE, 0);
+    gtk_widget_show(spinbutton);
+    xsane_back_gtk_set_tooltip(xsane.tooltips, spinbutton, DESC_SCALE_WIDTH); 
+
+    gtk_object_set_data(GTK_OBJECT(scalex_widget), "size-x-adjustment",  (void *) adjustment_size_x);
+    gtk_object_set_data(GTK_OBJECT(scalex_widget), "image_width",        (void *) image_info.image_width);
+    gtk_object_set_data(GTK_OBJECT(scalex_widget), "image_height",       (void *) image_info.image_height);
+
+    gtk_object_set_data(GTK_OBJECT(adjustment_size_x), "scale-adjustment",   (void *) scalex_widget);
+    gtk_object_set_data(GTK_OBJECT(adjustment_size_x), "image_width",        (void *) image_info.image_width);
+    gtk_object_set_data(GTK_OBJECT(adjustment_size_x), "image_height",       (void *) image_info.image_height);
+
+
+    /* Y */
+    hbox = gtk_hbox_new(FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(hbox), 4); 
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
+    gtk_widget_show(hbox);
+
     /* y_scale factor: <-> */
-    xsane_range_new_with_pixmap(xsane.xsane_window->window, GTK_BOX(vbox), zoom_y_xpm,
+    xsane_range_new_with_pixmap(xsane.xsane_window->window, GTK_BOX(hbox), zoom_y_xpm,
                                 DESC_Y_SCALE_FACTOR,
-                                0.01, 10.0, 0.01, 0.1, 2, &v->y_scale_factor, &scaley_widget,
-                                0, xsane_viewer_range_changed,
+                                0.01, 4.0, 0.01, 0.1, 2, &v->y_scale_factor, &scaley_widget,
+                                0, xsane_viewer_scale_set_scale_value_and_adjustments,
                                 TRUE);
+
+    /* y-size */
+    pixmap = gdk_pixmap_create_from_xpm_d(selection_dialog->window, &mask, xsane.bg_trans, (gchar **) size_y_xpm);
+    pixmapwidget = gtk_image_new_from_pixmap(pixmap, mask);
+    gtk_box_pack_start(GTK_BOX(hbox), pixmapwidget, FALSE, FALSE, 20);
+    gtk_widget_show(pixmapwidget);
+    gdk_drawable_unref(pixmap);
+
+    adjustment_size_y = (GtkAdjustment *) gtk_adjustment_new(v->y_scale_factor * image_info.image_height , 0.01 * image_info.image_height, 4.0 * image_info.image_height, 1.0, 5.0, 0.0);
+    spinbutton = gtk_spin_button_new(adjustment_size_y, 0, 0);
+    g_signal_connect(GTK_OBJECT(adjustment_size_y), "value_changed", (GtkSignalFunc) xsane_viewer_scale_set_size_y_value_and_adjustments, (void *) &v->y_scale_factor);
+    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spinbutton), FALSE);
+    gtk_widget_set_size_request(spinbutton, 80, -1);
+    gtk_box_pack_start(GTK_BOX(hbox), spinbutton, FALSE, FALSE, 0);
+    gtk_widget_show(spinbutton);
+    xsane_back_gtk_set_tooltip(xsane.tooltips, spinbutton, DESC_SCALE_HEIGHT); 
+
+    gtk_object_set_data(GTK_OBJECT(scaley_widget), "size-y-adjustment", (void *) adjustment_size_y);
+    gtk_object_set_data(GTK_OBJECT(scaley_widget), "image_width",       (void *) image_info.image_width);
+    gtk_object_set_data(GTK_OBJECT(scaley_widget), "image_height",      (void *) image_info.image_height);
+
+    gtk_object_set_data(GTK_OBJECT(adjustment_size_y), "scale-adjustment",   (void *) scaley_widget);
+    gtk_object_set_data(GTK_OBJECT(adjustment_size_y), "image_width",        (void *) image_info.image_width);
+    gtk_object_set_data(GTK_OBJECT(adjustment_size_y), "image_height",       (void *) image_info.image_height);
   }
 
   /* Apply Cancel */
@@ -686,7 +746,8 @@ static void xsane_viewer_despeckle_callback(GtkWidget *window, gpointer data)
 
   adjustment = (GtkAdjustment *) gtk_adjustment_new(2.0, 2.0, 10.0, 1.0, 5.0, 0.0);
   spinbutton = gtk_spin_button_new(adjustment, 0, 0);
-  g_signal_connect(GTK_OBJECT(spinbutton), DEF_GTK_SIGNAL_SPINBUTTON_VALUE_CHANGED, (GtkSignalFunc) xsane_viewer_spinbutton_int_changed, (void *) &v->despeckle_radius);
+//  g_signal_connect(GTK_OBJECT(spinbutton), DEF_GTK_SIGNAL_SPINBUTTON_VALUE_CHANGED, (GtkSignalFunc) xsane_viewer_spinbutton_int_changed, (void *) &v->despeckle_radius);
+  g_signal_connect(GTK_OBJECT(adjustment), "value_changed", (GtkSignalFunc) xsane_viewer_adjustment_int_changed, (void *) &v->despeckle_radius);
   gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spinbutton), TRUE);
   gtk_box_pack_end(GTK_BOX(hbox), spinbutton, FALSE, FALSE, 10);
   gtk_widget_show(spinbutton);
@@ -774,7 +835,8 @@ static void xsane_viewer_blur_callback(GtkWidget *window, gpointer data)
 
   adjustment = (GtkAdjustment *) gtk_adjustment_new(1.0, 1.0, 20.0, 0.1, 1.0, 0.0);
   spinbutton = gtk_spin_button_new(adjustment, 0, 2);
-  g_signal_connect(GTK_OBJECT(spinbutton), DEF_GTK_SIGNAL_SPINBUTTON_VALUE_CHANGED, (GtkSignalFunc) xsane_viewer_spinbutton_float_changed, (void *) &v->blur_radius);
+//  g_signal_connect(GTK_OBJECT(spinbutton), DEF_GTK_SIGNAL_SPINBUTTON_VALUE_CHANGED, (GtkSignalFunc) xsane_viewer_spinbutton_float_changed, (void *) &v->blur_radius);
+  g_signal_connect(GTK_OBJECT(adjustment), "value_changed", (GtkSignalFunc) xsane_viewer_adjustment_float_changed, (void *) &v->blur_radius);
   gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spinbutton), TRUE);
   gtk_box_pack_end(GTK_BOX(hbox), spinbutton, FALSE, FALSE, 10);
   gtk_widget_show(spinbutton);
@@ -1530,17 +1592,17 @@ Viewer *xsane_viewer_new(char *filename, int reduce_to_lineart, char *output_fil
   gtk_box_pack_start(GTK_BOX(vbox), v->button_box, FALSE, FALSE, 0);
   gtk_widget_show(v->button_box);
 
-  v->save      = xsane_button_new_with_pixmap(v->top->window, v->button_box, file_xpm,      DESC_VIEWER_SAVE,      (GtkSignalFunc) xsane_viewer_save_callback, v);
+  v->save      = xsane_button_new_with_pixmap(v->top->window, v->button_box, save_xpm,      DESC_VIEWER_SAVE,      (GtkSignalFunc) xsane_viewer_save_callback, v);
   v->ocr       = xsane_button_new_with_pixmap(v->top->window, v->button_box, ocr_xpm,       DESC_VIEWER_OCR,       (GtkSignalFunc) xsane_viewer_ocr_callback, v);
   v->clone     = xsane_button_new_with_pixmap(v->top->window, v->button_box, clone_xpm,     DESC_VIEWER_CLONE,     (GtkSignalFunc) xsane_viewer_clone_callback, v);
   v->scale     = xsane_button_new_with_pixmap(v->top->window, v->button_box, scale_xpm,     DESC_VIEWER_SCALE,     (GtkSignalFunc) xsane_viewer_scale_callback, v);
   v->despeckle = xsane_button_new_with_pixmap(v->top->window, v->button_box, despeckle_xpm, DESC_VIEWER_DESPECKLE, (GtkSignalFunc) xsane_viewer_despeckle_callback, v);
   v->blur      = xsane_button_new_with_pixmap(v->top->window, v->button_box, blur_xpm,      DESC_VIEWER_BLUR,      (GtkSignalFunc) xsane_viewer_blur_callback, v);
-  v->rotate90  = xsane_button_new_with_pixmap(v->top->window, v->button_box, rotate90_xpm,  DESC_VIEWER_ROTATE90,  (GtkSignalFunc) xsane_viewer_rotate90_callback, v);
-  v->rotate180 = xsane_button_new_with_pixmap(v->top->window, v->button_box, rotate180_xpm, DESC_VIEWER_ROTATE180, (GtkSignalFunc) xsane_viewer_rotate180_callback, v);
-  v->rotate270 = xsane_button_new_with_pixmap(v->top->window, v->button_box, rotate270_xpm, DESC_VIEWER_ROTATE270, (GtkSignalFunc) xsane_viewer_rotate270_callback, v);
-  v->mirror_x  = xsane_button_new_with_pixmap(v->top->window, v->button_box, mirror_x_xpm,  DESC_VIEWER_MIRROR_X,  (GtkSignalFunc) xsane_viewer_mirror_x_callback, v);
-  v->mirror_y  = xsane_button_new_with_pixmap(v->top->window, v->button_box, mirror_y_xpm,  DESC_VIEWER_MIRROR_Y,  (GtkSignalFunc) xsane_viewer_mirror_y_callback, v);
+  v->rotate90  = xsane_button_new_with_pixmap(v->top->window, v->button_box, rotate90_xpm,  DESC_ROTATE90,         (GtkSignalFunc) xsane_viewer_rotate90_callback, v);
+  v->rotate180 = xsane_button_new_with_pixmap(v->top->window, v->button_box, rotate180_xpm, DESC_ROTATE180,        (GtkSignalFunc) xsane_viewer_rotate180_callback, v);
+  v->rotate270 = xsane_button_new_with_pixmap(v->top->window, v->button_box, rotate270_xpm, DESC_ROTATE270,        (GtkSignalFunc) xsane_viewer_rotate270_callback, v);
+  v->mirror_x  = xsane_button_new_with_pixmap(v->top->window, v->button_box, mirror_x_xpm,  DESC_MIRROR_X,         (GtkSignalFunc) xsane_viewer_mirror_x_callback, v);
+  v->mirror_y  = xsane_button_new_with_pixmap(v->top->window, v->button_box, mirror_y_xpm,  DESC_MIRROR_Y,         (GtkSignalFunc) xsane_viewer_mirror_y_callback, v);
 
 
   /* "Zoom" submenu: */
@@ -1551,7 +1613,8 @@ Viewer *xsane_viewer_new(char *filename, int reduce_to_lineart, char *output_fil
 
   zoom_menu = gtk_menu_new();
   selection = 0;
-  for (i = 0; i < XSANE_VIEWER_ZOOM_ITEMS; i++)
+
+  for (i = 0; i < sizeof(xsane_viewer_zoom) / sizeof(int); i++)
   {
     snprintf(buf, sizeof(buf), "%d %%", xsane_viewer_zoom[i]);
     zoom_menu_item = gtk_menu_item_new_with_label(buf);
