@@ -27,13 +27,6 @@
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
-#if 0
-# define HAVE_GTK2
-# define HAVE_GTK_TEXT_VIEW_H
-#endif
-
-/* ---------------------------------------------------------------------------------------------------------------------- */
-
 #if 0
 #define DEF_GTK_ACCEL_LOCKED 0
 #else
@@ -42,21 +35,12 @@
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
-#ifndef HAVE_GTK2
-# include "xsane-gtk-1_x-compat.h"
-#else /* we have gtk+-2.0 */
-# define DEF_GTK_SIGNAL_SPINBUTTON_VALUE_CHANGED "value-changed"
-# define DEF_GTK_MENU_ACCEL_VISIBLE GTK_ACCEL_VISIBLE
-#endif
-
-/* ---------------------------------------------------------------------------------------------------------------------- */
-
 /* #define XSANE_TEST */
 /* #define SUPPORT_RGBA */
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
-#define XSANE_VERSION		"0.86"
+#define XSANE_VERSION		"0.90"
 #define XSANE_AUTHOR		"Oliver Rauch"
 #define XSANE_COPYRIGHT		"Oliver Rauch"
 #define XSANE_DATE		"1998-2002"
@@ -72,6 +56,7 @@
 #define XSANE_HOLD_TIME 200
 #define XSANE_CONTINUOUS_HOLD_TIME 10
 #define XSANE_DEFAULT_DEVICE "SANE_DEFAULT_DEVICE"
+#define XSANE_3PASS_BUFFER_RGB_SIZE 1024
 
 #ifndef SLASH
 # define SLASH '/'
@@ -97,7 +82,7 @@
 # include <lalloca.h>
 #endif
 
-#ifdef __hpux
+#if defined(__hpux) || defined(__sgi)
 # include <alloca.h>
 #endif
 
@@ -116,13 +101,15 @@
 #include <getopt.h>
 #include <time.h>
 
-#include <sys/stat.h>
+/* OS/2 want sys/types before sys/stat */
 #include <sys/types.h>
+#include <sys/stat.h>
 
 #include <locale.h>
 
 #include <sane/sane.h>
 #include <sane/saneopts.h>
+#include "xsaneopts.h"
 
 #include "../include/config.h"
 #include "../include/sanei_signal.h"
@@ -136,6 +123,21 @@
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
 
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
+#if GTK_MAJOR_VERSION == 2
+# define HAVE_GTK2
+#endif
+
+#ifdef HAVE_GTK2
+# define HAVE_GTK_TEXT_VIEW_H
+# define DEF_GTK_MENU_ACCEL_VISIBLE GTK_ACCEL_VISIBLE
+#else /* we don't have gtk+-2.0 */
+# include "xsane-gtk-1_x-compat.h"
+#endif
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
 #ifdef ENABLE_NLS
 #    include <libintl.h>
 #    define _(String) gettext (String)
@@ -157,6 +159,12 @@
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
+#ifdef HAVE_OS2_H
+# define strcasecmp stricmp
+#endif
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
 #ifdef HAVE_LIBPNG
 #ifdef HAVE_LIBZ
 # define XSANE_ACTIVATE_MAIL
@@ -184,6 +192,7 @@ typedef struct
 } pref_default_preset_area_t;
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
+
 typedef struct
   {
     /* The option number of the well-known options.  Each of these may
@@ -209,6 +218,10 @@ typedef struct
     int shadow_r;
     int shadow_g;
     int shadow_b;
+    int batch_scan_start;
+    int batch_scan_loop;
+    int batch_scan_end;
+    int batch_scan_next_tl_y;
   }
 GSGWellKnownOptions;
 
@@ -308,6 +321,7 @@ extern void xsane_pref_save(void);
 extern void xsane_interface(int argc, char **argv);
 extern void xsane_fax_project_save(void);
 extern void xsane_mail_project_save(void);
+extern void xsane_batch_scan_add(void);
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
@@ -337,8 +351,8 @@ extern void xsane_mail_project_save(void);
 #define OCRCOMMAND 	 	"gocr"
 #define OCRINPUTFILEOPT	 	"-i"
 #define OCROUTPUTFILEOPT	"-o"
-#define OCROUTFDOPT		"--gui-fdout"
-#define OCRPROGRESSKEY		"Progress:"
+#define OCROUTFDOPT		"-x"
+#define OCRPROGRESSKEY		""
 #define DOCVIEWER_NETSCAPE	"netscape"
 #define DOCVIEWER 	 	DOCVIEWER_NETSCAPE
 
@@ -398,7 +412,7 @@ extern void xsane_mail_project_save(void);
 enum
 {
  XSANE_UNKNOWN, XSANE_PNM, XSANE_JPEG, XSANE_PNG, XSANE_PS, XSANE_TIFF, XSANE_RGBA,
- XSANE_RAW16, XSANE_PNM16
+ XSANE_RAW16, XSANE_PNM16, XSANE_TEXT
 };
  
 /* ---------------------------------------------------------------------------------------------------------------------- */
@@ -431,12 +445,20 @@ typedef struct XsaneSlider
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
+typedef struct XsaneChildprocess
+{
+  pid_t pid;
+  struct XsaneChildprocess *next;
+} XsaneChildprocess;
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
 typedef struct Xsane
 {
     SANE_Int sane_backend_versioncode;
     char *backend;
+    char *backend_translation;
     char *device_set_filename;
-    char *filetype;
     char *output_filename;
     char *dummy_filename;
 
@@ -466,6 +488,11 @@ typedef struct Xsane
     int scanning;
     int reading_data;
     int cancel_scan;
+    int batch_loop; /* is set when batch scanning and not last scan */
+    int batch_scan_use_stored_scanmode;
+    int batch_scan_use_stored_resolution;
+    int batch_scan_use_stored_bit_depth;
+    SANE_Status status_of_last_scan;
 
 /* free gamma curve widgets */
     GtkWidget *gamma_curve_gray;
@@ -492,6 +519,12 @@ typedef struct Xsane
     GtkWidget *histogram_dialog;
     GtkWidget *gamma_dialog;
 
+    GtkWidget *batch_scan_dialog;
+    GtkWidget *batch_scan_button_box;
+    GtkWidget *batch_scan_action_box;
+    GtkWidget *batch_scan_list;
+    GtkAdjustment *batch_scan_vadjustment;
+
     GtkWidget *fax_dialog;
     GtkWidget *fax_list;
     GtkWidget *fax_project_box;
@@ -540,10 +573,12 @@ typedef struct Xsane
     size_t bytes_read;
     int read_offset_16;
     char last_offset_16_byte;
+    int  lineart_to_grayscale_x;
     GtkProgressBar *progress_bar;
     int input_tag;
     SANE_Parameters param;
     int adf_page_counter;
+    int scan_rotation;
 
     /* for standalone mode: */
     GtkWidget *filename_entry;
@@ -565,6 +600,7 @@ typedef struct Xsane
     struct XsaneSlider slider_red;
     struct XsaneSlider slider_green;
     struct XsaneSlider slider_blue;
+    guint  batch_scan_gamma_timer; /* has to be guint */
     guint  slider_timer; /* has to be guint */
     int    slider_timer_restart;
 
@@ -623,6 +659,7 @@ typedef struct Xsane
     GtkWidget *show_preview_widget;
     GtkWidget *show_histogram_widget;
     GtkWidget *show_gamma_widget;
+    GtkWidget *show_batch_scan_widget;
     GtkWidget *show_standard_options_widget;
     GtkWidget *show_advanced_options_widget;
     GtkWidget *show_resolution_list_widget;
@@ -751,6 +788,9 @@ typedef struct Xsane
 
 /* -------------------------------------------------- */
 
+    int ipc_pipefd[2]; /* for inter process communication error messages */
+    XsaneChildprocess *childprocess_list;
+
 } Xsane;
 
 extern struct Xsane xsane;
@@ -780,6 +820,7 @@ typedef struct XsaneSetup
   GtkWidget *overwrite_warning_button;
   GtkWidget *increase_filename_counter_button;
   GtkWidget *skip_existing_numbers_button;
+  GtkWidget *save_pnm16_as_ascii_button;
   GtkWidget *reduce_16bit_to_8bit_button;
 
   GtkWidget *main_window_fixed_button;
